import builtins
from _typeshed import Incomplete, Unused
from collections.abc import Callable, Collection, Sequence
from typing import Any, ClassVar, Final, Literal, NoReturn, SupportsIndex, TypeVar, overload
from typing_extensions import Self, TypeAlias, deprecated

import numpy as np
import pandas as pd
from numpy.typing import ArrayLike, DTypeLike, NDArray
from pandas.api.extensions import ExtensionArray, ExtensionDtype
from pyproj import CRS, Transformer
from shapely import Geometry
from shapely.geometry.base import BaseGeometry

from .base import _AffinityOrigin, _ConvertibleToCRS
from .sindex import SpatialIndex

_ScalarType = TypeVar("_ScalarType", bound=np.generic)
_Array1D: TypeAlias = np.ndarray[tuple[int], np.dtype[_ScalarType]]
_Array2D: TypeAlias = np.ndarray[tuple[int, int], np.dtype[_ScalarType]]
_ArrayOrGeom: TypeAlias = GeometryArray | ArrayLike | Geometry

TransformerFromCRS = Transformer.from_crs
POLYGON_GEOM_TYPES: Final[set[str]]
LINE_GEOM_TYPES: Final[set[str]]
POINT_GEOM_TYPES: Final[set[str]]

class GeometryDtype(ExtensionDtype):
    type: ClassVar[type[BaseGeometry]]
    name: ClassVar[str]
    na_value: None
    @classmethod
    def construct_from_string(cls, string: str) -> Self: ...
    @classmethod
    def construct_array_type(cls) -> builtins.type[GeometryArray]: ...

def isna(value: object) -> bool: ...
def from_shapely(data, crs: _ConvertibleToCRS | None = None) -> GeometryArray: ...
def to_shapely(geoms: GeometryArray) -> _Array1D[np.object_]: ...
def from_wkb(
    data, crs: _ConvertibleToCRS | None = None, on_invalid: Literal["raise", "warn", "ignore", "fix"] = "raise"
) -> GeometryArray: ...
@overload
def to_wkb(geoms: GeometryArray, hex: Literal[False] = False, **kwargs) -> _Array1D[np.bytes_]: ...
@overload
def to_wkb(geoms: GeometryArray, hex: Literal[True], **kwargs) -> _Array1D[np.str_]: ...
def from_wkt(
    data, crs: _ConvertibleToCRS | None = None, on_invalid: Literal["raise", "warn", "ignore", "fix"] = "raise"
) -> GeometryArray: ...
def to_wkt(geoms: GeometryArray, **kwargs) -> _Array1D[np.str_]: ...
def points_from_xy(
    x: ArrayLike, y: ArrayLike, z: ArrayLike | None = None, crs: _ConvertibleToCRS | None = None
) -> GeometryArray: ...

class GeometryArray(ExtensionArray):
    def __init__(self, data: GeometryArray | NDArray[np.object_], crs: _ConvertibleToCRS | None = None) -> None: ...
    @property
    def sindex(self) -> SpatialIndex: ...
    @property
    def has_sindex(self) -> bool: ...
    @property
    def crs(self) -> CRS | None: ...
    @crs.setter
    def crs(self, value: _ConvertibleToCRS | None) -> None: ...
    def check_geographic_crs(self, stacklevel: int) -> None: ...
    @property
    def dtype(self) -> GeometryDtype: ...
    def __len__(self) -> int: ...
    # np.integer[Any] because precision is not important
    @overload
    def __getitem__(self, idx: int | np.integer[Any]) -> BaseGeometry: ...  # Always 1-D, doesn't accept tuple
    @overload
    def __getitem__(
        self, idx: slice | Sequence[SupportsIndex] | NDArray[np.bool_] | NDArray[np.integer[Any]]
    ) -> GeometryArray: ...
    @overload
    def __getitem__(
        self, idx: int | np.integer[Any] | slice | Sequence[int] | NDArray[np.bool_] | NDArray[np.integer[Any]]
    ) -> BaseGeometry | GeometryArray: ...
    def __setitem__(
        self, key, value: _ArrayOrGeom | pd.DataFrame | pd.Series[Any]  # Cannot use pd.Series[BaseGeometry]
    ) -> None: ...
    @property
    def is_valid(self) -> _Array1D[np.bool_]: ...
    def is_valid_reason(self) -> _Array1D[np.object_]: ...
    def is_valid_coverage(self, gap_width: float = 0.0) -> bool: ...
    def invalid_coverage_edges(self, gap_width: float = 0.0) -> _Array1D[np.object_]: ...
    @property
    def is_empty(self) -> _Array1D[np.bool_]: ...
    @property
    def is_simple(self) -> _Array1D[np.bool_]: ...
    @property
    def is_ring(self) -> _Array1D[np.bool_]: ...
    @property
    def is_closed(self) -> _Array1D[np.bool_]: ...
    @property
    def is_ccw(self) -> _Array1D[np.bool_]: ...
    @property
    def has_z(self) -> _Array1D[np.bool_]: ...
    @property
    def has_m(self) -> _Array1D[np.bool_]: ...
    @property
    def geom_type(self) -> _Array1D[np.str_]: ...
    @property
    def area(self) -> _Array1D[np.float64]: ...
    @property
    def length(self) -> _Array1D[np.float64]: ...
    def count_coordinates(self) -> _Array1D[np.int32]: ...
    def count_geometries(self) -> _Array1D[np.int32]: ...
    def count_interior_rings(self) -> _Array1D[np.int32]: ...
    def get_precision(self) -> _Array1D[np.float64]: ...
    def get_geometry(self, index: SupportsIndex | ArrayLike) -> _Array1D[np.object_]: ...
    @property
    def boundary(self) -> GeometryArray: ...
    @property
    def centroid(self) -> GeometryArray: ...
    def concave_hull(self, ratio: float, allow_holes: bool) -> _Array1D[np.object_]: ...
    def constrained_delaunay_triangles(self) -> GeometryArray: ...
    @property
    def convex_hull(self) -> GeometryArray: ...
    @property
    def envelope(self) -> GeometryArray: ...
    def minimum_rotated_rectangle(self) -> GeometryArray: ...
    @property
    def exterior(self) -> GeometryArray: ...
    def extract_unique_points(self) -> GeometryArray: ...
    def offset_curve(
        self,
        distance: float | ArrayLike,
        quad_segs: int = 8,
        join_style: Literal["round", "bevel", "mitre"] = "round",
        mitre_limit: float = 5.0,
    ) -> GeometryArray: ...
    @property
    def interiors(self) -> _Array1D[np.object_]: ...
    def remove_repeated_points(self, tolerance: float | ArrayLike = 0.0) -> GeometryArray: ...
    def representative_point(self) -> GeometryArray: ...
    def minimum_bounding_circle(self) -> GeometryArray: ...
    def maximum_inscribed_circle(self, tolerance: float | ArrayLike) -> GeometryArray: ...
    def minimum_bounding_radius(self) -> _Array1D[np.float64]: ...
    def minimum_clearance(self) -> _Array1D[np.float64]: ...
    def minimum_clearance_line(self) -> GeometryArray: ...
    def normalize(self) -> GeometryArray: ...
    def orient_polygons(self, exterior_cw: bool = False) -> GeometryArray: ...
    def make_valid(self, method: Literal["linework", "structure"] = "linework", keep_collapsed: bool = True) -> GeometryArray: ...
    def reverse(self) -> GeometryArray: ...
    def segmentize(self, max_segment_length: float | ArrayLike) -> GeometryArray: ...
    def force_2d(self) -> GeometryArray: ...
    def force_3d(self, z: float | ArrayLike = 0) -> GeometryArray: ...
    def transform(
        self, transformation: Callable[[NDArray[np.float64]], NDArray[np.float64]], include_z: bool = False
    ) -> GeometryArray: ...
    def line_merge(self, directed: bool = False) -> GeometryArray: ...
    def set_precision(
        self, grid_size: float, mode: Literal["valid_output", "pointwise", "keep_collapsed", 0, 1, 2] = "valid_output"
    ) -> GeometryArray: ...
    def covers(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def covered_by(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def contains(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def contains_properly(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def crosses(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def disjoint(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def geom_equals(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def intersects(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def overlaps(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def touches(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def within(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def dwithin(self, other: _ArrayOrGeom, distance: float) -> _Array1D[np.bool_]: ...
    def geom_equals_exact(self, other: _ArrayOrGeom, tolerance: float | ArrayLike) -> _Array1D[np.bool_]: ...
    def geom_equals_identical(self, other: _ArrayOrGeom) -> _Array1D[np.bool_]: ...
    def clip_by_rect(self, xmin: float, ymin: float, xmax: float, ymax: float) -> GeometryArray: ...
    def difference(self, other: _ArrayOrGeom) -> GeometryArray: ...
    def intersection(self, other: _ArrayOrGeom) -> GeometryArray: ...
    def symmetric_difference(self, other: _ArrayOrGeom) -> GeometryArray: ...
    def union(self, other: _ArrayOrGeom) -> GeometryArray: ...
    def shortest_line(self, other: _ArrayOrGeom) -> GeometryArray: ...
    def snap(self, other: _ArrayOrGeom, tolerance: float | ArrayLike) -> GeometryArray: ...
    def shared_paths(self, other: _ArrayOrGeom) -> GeometryArray: ...
    def distance(self, other: _ArrayOrGeom) -> _Array1D[np.float64]: ...
    def hausdorff_distance(self, other: _ArrayOrGeom, **kwargs) -> _Array1D[np.float64]: ...
    def frechet_distance(self, other: _ArrayOrGeom, **kwargs) -> _Array1D[np.float64]: ...
    def buffer(self, distance: float | ArrayLike, resolution: int = 16, **kwargs) -> GeometryArray: ...
    def interpolate(self, distance: float | ArrayLike, normalized: bool = False) -> GeometryArray: ...
    def simplify(self, tolerance: float | ArrayLike, preserve_topology: bool = True) -> GeometryArray: ...
    def simplify_coverage(self, tolerance: float | ArrayLike, simplify_boundary: bool = True) -> GeometryArray: ...
    def project(self, other: _ArrayOrGeom, normalized: bool = False) -> _Array1D[np.float64]: ...
    def relate(self, other: _ArrayOrGeom) -> _Array1D[np.str_]: ...
    def relate_pattern(self, other: _ArrayOrGeom, pattern: str) -> _Array1D[np.bool_]: ...
    @deprecated("Use method `union_all` instead.")
    def unary_union(self) -> BaseGeometry: ...
    def union_all(
        self, method: Literal["coverage", "unary", "disjoint_subset"] = "unary", grid_size: float | None = None
    ) -> BaseGeometry: ...
    def intersection_all(self) -> BaseGeometry: ...
    def affine_transform(self, matrix: Collection[float]) -> GeometryArray: ...
    def translate(self, xoff: float = 0.0, yoff: float = 0.0, zoff: float = 0.0) -> GeometryArray: ...
    def rotate(self, angle: float, origin: _AffinityOrigin = "center", use_radians: bool = False) -> GeometryArray: ...
    def scale(
        self, xfact: float = 1.0, yfact: float = 1.0, zfact: float = 1.0, origin: _AffinityOrigin = "center"
    ) -> GeometryArray: ...
    def skew(
        self, xs: float = 0.0, ys: float = 0.0, origin: _AffinityOrigin = "center", use_radians: bool = False
    ) -> GeometryArray: ...
    def to_crs(self, crs: _ConvertibleToCRS | None = None, epsg: int | None = None) -> GeometryArray: ...
    def estimate_utm_crs(self, datum_name: str = "WGS 84") -> CRS: ...
    @property
    def x(self) -> _Array1D[np.float64]: ...
    @property
    def y(self) -> _Array1D[np.float64]: ...
    @property
    def z(self) -> _Array1D[np.float64]: ...
    @property
    def m(self) -> _Array1D[np.float64]: ...
    @property
    def bounds(self) -> _Array2D[np.float64]: ...
    @property
    def total_bounds(self) -> _Array1D[np.float64]: ...
    @property
    def size(self) -> int: ...
    @property
    def shape(self) -> tuple[int]: ...  # Always 1-D, this is not mistaken for tuple[int, ...]
    @property
    def ndim(self) -> Literal[1]: ...
    def copy(self, *args: Unused, **kwargs: Unused) -> GeometryArray: ...
    def take(
        self, indices: Sequence[SupportsIndex] | NDArray[np.integer], allow_fill: bool = False, fill_value: Geometry | None = None
    ) -> GeometryArray: ...
    def fillna(
        self,
        value: Geometry | GeometryArray | None = None,
        method: Literal["backfill", "bfill", "pad", "ffill"] | None = None,
        limit: int | None = None,
        copy: bool = True,
    ) -> GeometryArray: ...
    @overload
    def astype(self, dtype: GeometryDtype, copy: bool = True) -> GeometryArray: ...
    @overload
    def astype(self, dtype: ExtensionDtype | Literal["string"], copy: bool = True) -> ExtensionArray: ...  # type: ignore[overload-overlap]
    @overload
    def astype(self, dtype: DTypeLike, copy: bool = True) -> _Array1D[Incomplete]: ...
    def isna(self) -> _Array1D[np.bool_]: ...
    def value_counts(self, dropna: bool = True) -> pd.Series[int]: ...
    def unique(self) -> GeometryArray: ...
    @property
    def nbytes(self) -> int: ...
    def shift(self, periods: int = 1, fill_value: Geometry | None = None) -> GeometryArray: ...  # type: ignore[override]
    def argmin(self, skipna: bool = True) -> NoReturn: ...
    def argmax(self, skipna: bool = True) -> NoReturn: ...
    def __array__(self, dtype: DTypeLike | None = None, copy: bool | None = None) -> _Array1D[np.object_]: ...
    def __eq__(self, other: object) -> _Array1D[np.bool_]: ...  # type: ignore[override]
    def __ne__(self, other: object) -> _Array1D[np.bool_]: ...  # type: ignore[override]
    def __contains__(self, item: object) -> np.bool_: ...

# TODO: Improve `func` type with a callable protocol (with overloads for 2D and 3D geometries)
def transform(
    data: NDArray[np.object_] | GeometryArray | pd.Series[Any],  # Cannot use pd.Series[BaseGeometry]
    func: Callable[..., NDArray[np.float64]],
) -> NDArray[np.object_]: ...



from _typeshed import Incomplete
from types import TracebackType
from typing import Final

from ..recorder import AWSXRayRecorder
from ..utils.atomic_counter import AtomicCounter
from .dummy_entities import DummySegment
from .entity import Entity
from .subsegment import Subsegment

ORIGIN_TRACE_HEADER_ATTR_KEY: Final = "_origin_trace_header"

class SegmentContextManager:
    name: str | None
    segment_kwargs: dict[str, str | bool | None]
    recorder: AWSXRayRecorder
    segment: Segment | None
    def __init__(
        self,
        recorder: AWSXRayRecorder,
        name: str | None = None,
        *,
        traceid: str | None = None,
        parent_id: str | None = None,
        sampling: bool | None = None,
    ) -> None: ...
    def __enter__(self) -> DummySegment | Segment: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...

class Segment(Entity):
    trace_id: str
    id: str
    in_progress: bool
    sampled: bool
    user: str | None
    ref_counter: AtomicCounter
    parent_id: str
    service: dict[str, str]
    def __init__(
        self,
        name: str,
        entityid: str | None = None,
        traceid: str | None = None,
        parent_id: str | None = None,
        sampled: bool = True,
    ) -> None: ...
    def add_subsegment(self, subsegment: Subsegment) -> None: ...
    def increment(self) -> None: ...
    def decrement_ref_counter(self) -> None: ...
    def ready_to_send(self) -> bool: ...
    def get_total_subsegments_size(self) -> int: ...
    def decrement_subsegments_size(self) -> int: ...
    def remove_subsegment(self, subsegment: Subsegment) -> None: ...
    def set_user(self, user) -> None: ...
    def set_service(self, service_info: dict[str, str]) -> None: ...
    def set_rule_name(self, rule_name: str) -> None: ...
    def to_dict(self) -> dict[str, Incomplete]: ...



from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["boykov_kolmogorov"]

@_dispatchable
def boykov_kolmogorov(
    G: Graph[_Node],
    s: _Node,
    t: _Node,
    capacity: str = "capacity",
    residual: Graph[_Node] | None = None,
    value_only: bool = False,
    cutoff: float | None = None,
): ...



"""Constants across the project."""

from enum import Enum, IntEnum, IntFlag, StrEnum


class SymbolType(Enum):
    """Symbol types."""

    MODULE = "module"  # LSP: Module
    MOD_VAR = "mod_var"  # LSP: Variable
    VAR = "variable"  # LSP: Variable
    IMM_VAR = "immutable"  # LSP: Constant
    ABILITY = "ability"  # LSP: Function
    OBJECT_ARCH = "object"  # LSP: Class
    NODE_ARCH = "node"  # LSP: Class
    EDGE_ARCH = "edge"  # LSP: Class
    WALKER_ARCH = "walker"  # LSP: Class
    ENUM_ARCH = "enum"  # LSP: Enum
    TEST = "test"  # LSP: Function
    TYPE = "type"  # LSP: TypeParameter
    IMPL = "impl"  # LSP: Interface or Property
    SEM = "sem"  # LSP: Property
    HAS_VAR = "field"  # LSP: Field
    METHOD = "method"  # LSP: Method
    CONSTRUCTOR = "constructor"  # LSP: Constructor
    ENUM_MEMBER = "enum_member"  # LSP: EnumMember
    NUMBER = "number"  # LSP: Number
    STRING = "string"  # LSP: String
    BOOL = "bool"  # LSP: Boolean
    SEQUENCE = "sequence"  # LSP: Array
    NULL = "null"  # LSP: Null
    UNKNOWN = "unknown"  # LSP: Unknown

    def __str__(self) -> str:
        return self.value


class JacSemTokenType(IntEnum):
    """LSP Token types for Jac."""

    NAMESPACE = 0
    TYPE = 1
    CLASS = 2
    ENUM = 3
    INTERFACE = 4
    STRUCT = 5
    TYPE_PARAMETER = 6
    PARAMETER = 7
    VARIABLE = 8
    PROPERTY = 9
    ENUM_MEMBER = 10
    EVENT = 11
    FUNCTION = 12
    METHOD = 13
    MACRO = 14
    KEYWORD = 15
    MODIFIER = 16
    COMMENT = 17
    STRING = 18
    NUMBER = 19
    REGEXP = 20
    OPERATOR = 21

    @staticmethod
    def as_str_list() -> list[str]:
        return [i.name.lower() for i in JacSemTokenType]


class JacSemTokenModifier(IntFlag):
    """LSP Token modifiers for Jac."""

    DECLARATION = 1 << 0
    DEFINITION = 1 << 1
    READONLY = 1 << 2
    STATIC = 1 << 3
    DEPRECATED = 1 << 4
    ABSTRACT = 1 << 5
    ASYNC = 1 << 6
    MODIFICATION = 1 << 7
    DOCUMENTATION = 1 << 8
    DEFAULT_LIBRARY = 1 << 9

    @staticmethod
    def as_str_list() -> list[str]:
        """Return the string representation of the token."""
        return [i.name.lower() for i in JacSemTokenModifier if i.name]


class Constants(StrEnum):
    """Token constants for Jac."""

    HERE = "here"
    ROOT = "root"
    VISITOR = "visitor"
    JAC_CHECK = "_check"
    SUPER_ROOT_UUID = "00000000-0000-0000-0000-000000000000"

    def __str__(self) -> str:
        """Return the string representation of the token."""
        return self.value


class CodeContext(Enum):
    """Code execution context for client/server separation."""

    SERVER = "server"  # Default: runs on server (Python)
    CLIENT = "client"  # Runs on client (JavaScript/browser)

    def __str__(self) -> str:
        """Return the string representation of the context."""
        return self.value

    @property
    def is_server(self) -> bool:
        """Check if this is server context."""
        return self == CodeContext.SERVER

    @property
    def is_client(self) -> bool:
        """Check if this is client context."""
        return self == CodeContext.CLIENT


class EdgeDir(Enum):
    """Edge direction indicator."""

    IN = 1  # <--
    OUT = 2  # -->
    ANY = 3  # <-->


class SymbolAccess(Enum):
    """Symbol types."""

    PRIVATE = "private"
    PUBLIC = "public"
    PROTECTED = "protected"

    def __str__(self) -> str:
        """Stringify."""
        return self.value


# Done like this for type checker
# validated synced with test
class Tokens(str, Enum):
    """Token constants for the lexer."""

    FLOAT = "FLOAT"
    STRING = "STRING"
    PYNLINE = "PYNLINE"
    BOOL = "BOOL"
    INT = "INT"
    HEX = "HEX"
    BIN = "BIN"
    OCT = "OCT"
    NULL = "NULL"
    NAME = "NAME"
    KWESC_NAME = "KWESC_NAME"
    TYP_STRING = "TYP_STRING"
    TYP_INT = "TYP_INT"
    TYP_FLOAT = "TYP_FLOAT"
    TYP_LIST = "TYP_LIST"
    TYP_TUPLE = "TYP_TUPLE"
    TYP_SET = "TYP_SET"
    TYP_DICT = "TYP_DICT"
    TYP_BOOL = "TYP_BOOL"
    TYP_BYTES = "TYP_BYTES"
    TYP_ANY = "TYP_ANY"
    TYP_TYPE = "TYP_TYPE"
    KW_ABSTRACT = "KW_ABSTRACT"
    KW_OBJECT = "KW_OBJECT"
    KW_CLASS = "KW_CLASS"
    KW_ENUM = "KW_ENUM"
    KW_NODE = "KW_NODE"
    KW_VISIT = "KW_VISIT"
    KW_SPAWN = "KW_SPAWN"
    KW_WITH = "KW_WITH"
    KW_LAMBDA = "KW_LAMBDA"
    KW_ENTRY = "KW_ENTRY"
    KW_EXIT = "KW_EXIT"
    KW_IMPORT = "KW_IMPORT"
    KW_INCLUDE = "KW_INCLUDE"
    KW_FROM = "KW_FROM"
    KW_AS = "KW_AS"
    KW_EDGE = "KW_EDGE"
    KW_WALKER = "KW_WALKER"
    KW_ASYNC = "KW_ASYNC"
    KW_AWAIT = "KW_AWAIT"
    KW_FLOW = "KW_FLOW"
    KW_WAIT = "KW_WAIT"
    KW_TEST = "KW_TEST"
    KW_ASSERT = "KW_ASSERT"
    COLON = "COLON"
    PIPE_FWD = "PIPE_FWD"
    PIPE_BKWD = "PIPE_BKWD"
    DOT_FWD = "DOT_FWD"
    DOT_BKWD = "DOT_BKWD"
    LBRACE = "LBRACE"
    RBRACE = "RBRACE"
    SEMI = "SEMI"
    EQ = "EQ"
    ADD_EQ = "ADD_EQ"
    SUB_EQ = "SUB_EQ"
    MUL_EQ = "MUL_EQ"
    STAR_POW_EQ = "STAR_POW_EQ"
    FLOOR_DIV_EQ = "FLOOR_DIV_EQ"
    DIV_EQ = "DIV_EQ"
    MOD_EQ = "MOD_EQ"
    BW_AND_EQ = "BW_AND_EQ"
    BW_OR_EQ = "BW_OR_EQ"
    BW_XOR_EQ = "BW_XOR_EQ"
    LSHIFT_EQ = "LSHIFT_EQ"
    RSHIFT_EQ = "RSHIFT_EQ"
    WALRUS_EQ = "WALRUS_EQ"
    MATMUL_EQ = "MATMUL_EQ"
    KW_AND = "KW_AND"
    KW_OR = "KW_OR"
    KW_IF = "KW_IF"
    KW_ELIF = "KW_ELIF"
    KW_ELSE = "KW_ELSE"
    KW_FOR = "KW_FOR"
    KW_TO = "KW_TO"
    KW_BY = "KW_BY"
    KW_WHILE = "KW_WHILE"
    KW_CONTINUE = "KW_CONTINUE"
    KW_BREAK = "KW_BREAK"
    KW_DISENGAGE = "KW_DISENGAGE"
    KW_YIELD = "KW_YIELD"
    KW_SKIP = "KW_SKIP"
    KW_REPORT = "KW_REPORT"
    KW_RETURN = "KW_RETURN"
    KW_DELETE = "KW_DELETE"
    KW_TRY = "KW_TRY"
    KW_EXCEPT = "KW_EXCEPT"
    KW_FINALLY = "KW_FINALLY"
    KW_RAISE = "KW_RAISE"
    ELLIPSIS = "ELLIPSIS"
    DOT = "DOT"
    NOT = "NOT"
    EE = "EE"
    LT = "LT"
    GT = "GT"
    LTE = "LTE"
    GTE = "GTE"
    NE = "NE"
    KW_IN = "KW_IN"
    KW_IS = "KW_IS"
    KW_NIN = "KW_NIN"
    KW_ISN = "KW_ISN"
    KW_PRIV = "KW_PRIV"
    KW_PUB = "KW_PUB"
    KW_PROT = "KW_PROT"
    KW_HAS = "KW_HAS"
    KW_GLOBAL = "KW_GLOBAL"
    COMMA = "COMMA"
    KW_CAN = "KW_CAN"
    KW_DEF = "KW_DEF"
    KW_STATIC = "KW_STATIC"
    KW_OVERRIDE = "KW_OVERRIDE"
    KW_MATCH = "KW_MATCH"
    KW_SWITCH = "KW_SWITCH"
    KW_CASE = "KW_CASE"
    KW_DEFAULT = "KW_DEFAULT"
    KW_CLIENT = "KW_CLIENT"
    KW_SERVER = "KW_SERVER"
    PLUS = "PLUS"
    MINUS = "MINUS"
    STAR_MUL = "STAR_MUL"
    FLOOR_DIV = "FLOOR_DIV"
    DIV = "DIV"
    MOD = "MOD"
    BW_AND = "BW_AND"
    BW_OR = "BW_OR"
    BW_XOR = "BW_XOR"
    BW_NOT = "BW_NOT"
    LSHIFT = "LSHIFT"
    RSHIFT = "RSHIFT"
    STAR_POW = "STAR_POW"
    LPAREN = "LPAREN"
    RPAREN = "RPAREN"
    LSQUARE = "LSQUARE"
    RSQUARE = "RSQUARE"
    ARROW_L = "ARROW_L"
    ARROW_R = "ARROW_R"
    ARROW_BI = "ARROW_BI"
    ARROW_L_P1 = "ARROW_L_P1"
    ARROW_L_P2 = "ARROW_L_P2"
    ARROW_R_P1 = "ARROW_R_P1"
    ARROW_R_P2 = "ARROW_R_P2"
    CARROW_L = "CARROW_L"
    CARROW_R = "CARROW_R"
    CARROW_BI = "CARROW_BI"
    CARROW_L_P1 = "CARROW_L_P1"
    CARROW_L_P2 = "CARROW_L_P2"
    CARROW_R_P1 = "CARROW_R_P1"
    CARROW_R_P2 = "CARROW_R_P2"
    GLOBAL_OP = "GLOBAL_OP"
    NONLOCAL_OP = "NONLOCAL_OP"
    KW_HERE = "KW_HERE"
    KW_VISITOR = "KW_VISITOR"
    KW_SELF = "KW_SELF"
    KW_PROPS = "KW_PROPS"
    KW_INIT = "KW_INIT"
    KW_SUPER = "KW_SUPER"
    KW_ROOT = "KW_ROOT"
    KW_POST_INIT = "KW_POST_INIT"
    KW_IMPL = "KW_IMPL"
    KW_SEM = "KW_SEM"
    TYPE_OP = "TYPE_OP"
    A_PIPE_FWD = "A_PIPE_FWD"
    A_PIPE_BKWD = "A_PIPE_BKWD"
    RETURN_HINT = "RETURN_HINT"
    NULL_OK = "NULL_OK"
    DECOR_OP = "DECOR_OP"
    JSX_TEXT = "JSX_TEXT"
    JSX_OPEN_START = "JSX_OPEN_START"
    JSX_SELF_CLOSE = "JSX_SELF_CLOSE"
    JSX_TAG_END = "JSX_TAG_END"
    JSX_CLOSE_START = "JSX_CLOSE_START"
    JSX_FRAG_OPEN = "JSX_FRAG_OPEN"
    JSX_FRAG_CLOSE = "JSX_FRAG_CLOSE"
    JSX_NAME = "JSX_NAME"
    COMMENT = "COMMENT"
    WS = "WS"
    F_DQ_START = "F_DQ_START"
    F_SQ_START = "F_SQ_START"
    F_TDQ_START = "F_TDQ_START"
    F_TSQ_START = "F_TSQ_START"
    RF_DQ_START = "RF_DQ_START"
    RF_SQ_START = "RF_SQ_START"
    RF_TDQ_START = "RF_TDQ_START"
    RF_TSQ_START = "RF_TSQ_START"
    F_DQ_END = "F_DQ_END"
    F_SQ_END = "F_SQ_END"
    F_TDQ_END = "F_TDQ_END"
    F_TSQ_END = "F_TSQ_END"
    F_TEXT_DQ = "F_TEXT_DQ"
    F_TEXT_SQ = "F_TEXT_SQ"
    F_TEXT_TDQ = "F_TEXT_TDQ"
    F_TEXT_TSQ = "F_TEXT_TSQ"
    RF_TEXT_DQ = "RF_TEXT_DQ"
    RF_TEXT_SQ = "RF_TEXT_SQ"
    RF_TEXT_TDQ = "RF_TEXT_TDQ"
    RF_TEXT_TSQ = "RF_TEXT_TSQ"
    D_LBRACE = "D_LBRACE"
    D_RBRACE = "D_RBRACE"
    CONV = "CONV"
    F_FORMAT_TEXT = "F_FORMAT_TEXT"

    def __str__(self) -> str:
        return self.value


DELIM_MAP = {
    Tokens.COMMA: ",",
    Tokens.EQ: "=",
    Tokens.DECOR_OP: "@",
    Tokens.WS: "\n",
    Tokens.SEMI: ";",
    Tokens.COLON: ":",
    Tokens.LBRACE: "{",
    Tokens.RBRACE: "}",
    Tokens.LSQUARE: "[",
    Tokens.RSQUARE: "]",
    Tokens.LPAREN: "(",
    Tokens.RPAREN: ")",
    Tokens.RETURN_HINT: "->",
    Tokens.DOT: ".",
}

colors = [
    "#FFE9E9",
    "#F0FFF0",
    "#F5E5FF",
    "#FFFFE0",
    "#D2FEFF",
    "#E8FFD7",
    "#FFDEAD",
    "#FFF0F5",
    "#F5FFFA",
    "#FFC0CB",
    "#7FFFD4",
    "#C0C0C0",
    "#ADD8E6",
    "#FFFAF0",
    "#f4f3f7",
    "#f5efff",
    "#b5d7fd",
    "#ffc0cb",
    "#FFC0CB",
    "#e1d4c0",
    "#FCDFFF",
    "#F0FFFF",
    "#F0F8FF",
    "#F8F8FF",
    "#F0FFFF",
]


# =============================================================================
# TypeScript/JavaScript Parser Constants
# =============================================================================


class TsTokens(str, Enum):
    """Token constants for the TypeScript/JavaScript lexer."""

    # Literals
    NUMBER = "NUMBER"
    STRING = "STRING"
    TEMPLATE_STRING = "TEMPLATE_STRING"
    TEMPLATE_HEAD = "TEMPLATE_HEAD"
    TEMPLATE_MIDDLE = "TEMPLATE_MIDDLE"
    TEMPLATE_TAIL = "TEMPLATE_TAIL"
    REGEX = "REGEX"
    TRUE = "TRUE"
    FALSE = "FALSE"
    NULL = "NULL"
    UNDEFINED = "UNDEFINED"

    # Identifiers
    NAME = "NAME"
    PRIVATE_NAME = "PRIVATE_NAME"  # #privateField

    # Keywords - Declarations
    KW_VAR = "KW_VAR"
    KW_CONST = "KW_CONST"
    KW_FUNCTION = "KW_FUNCTION"
    KW_CLASS = "KW_CLASS"
    KW_INTERFACE = "KW_INTERFACE"
    KW_TYPE = "KW_TYPE"
    KW_ENUM = "KW_ENUM"
    KW_NAMESPACE = "KW_NAMESPACE"
    KW_MODULE = "KW_MODULE"
    KW_DECLARE = "KW_DECLARE"

    # Keywords - Class/Interface
    KW_EXTENDS = "KW_EXTENDS"
    KW_IMPLEMENTS = "KW_IMPLEMENTS"
    KW_STATIC = "KW_STATIC"
    KW_PUBLIC = "KW_PUBLIC"
    KW_PRIVATE = "KW_PRIVATE"
    KW_PROTECTED = "KW_PROTECTED"
    KW_READONLY = "KW_READONLY"
    KW_ABSTRACT = "KW_ABSTRACT"
    KW_OVERRIDE = "KW_OVERRIDE"
    KW_CONSTRUCTOR = "KW_CONSTRUCTOR"
    KW_GET = "KW_GET"
    KW_SET = "KW_SET"

    # Keywords - Control Flow
    KW_IF = "KW_IF"
    KW_ELSE = "KW_ELSE"
    KW_SWITCH = "KW_SWITCH"
    KW_CASE = "KW_CASE"
    KW_DEFAULT = "KW_DEFAULT"
    KW_FOR = "KW_FOR"
    KW_WHILE = "KW_WHILE"
    KW_DO = "KW_DO"
    KW_BREAK = "KW_BREAK"
    KW_CONTINUE = "KW_CONTINUE"
    KW_RETURN = "KW_RETURN"
    KW_THROW = "KW_THROW"
    KW_TRY = "KW_TRY"
    KW_CATCH = "KW_CATCH"
    KW_FINALLY = "KW_FINALLY"
    KW_WITH = "KW_WITH"
    KW_DEBUGGER = "KW_DEBUGGER"

    # Keywords - Operators
    KW_NEW = "KW_NEW"
    KW_DELETE = "KW_DELETE"
    KW_TYPEOF = "KW_TYPEOF"
    KW_VOID = "KW_VOID"
    KW_IN = "KW_IN"
    KW_OF = "KW_OF"
    KW_INSTANCEOF = "KW_INSTANCEOF"

    # Keywords - Async
    KW_ASYNC = "KW_ASYNC"
    KW_AWAIT = "KW_AWAIT"
    KW_YIELD = "KW_YIELD"

    # Keywords - Module
    KW_IMPORT = "KW_IMPORT"
    KW_EXPORT = "KW_EXPORT"
    KW_FROM = "KW_FROM"
    KW_AS = "KW_AS"

    # Keywords - Special
    KW_THIS = "KW_THIS"
    KW_SUPER = "KW_SUPER"

    # TypeScript Type Keywords
    KW_ANY = "KW_ANY"
    KW_UNKNOWN = "KW_UNKNOWN"
    KW_NEVER = "KW_NEVER"
    KW_STRING_TYPE = "KW_STRING_TYPE"
    KW_NUMBER_TYPE = "KW_NUMBER_TYPE"
    KW_BOOLEAN_TYPE = "KW_BOOLEAN_TYPE"
    KW_SYMBOL_TYPE = "KW_SYMBOL_TYPE"
    KW_BIGINT_TYPE = "KW_BIGINT_TYPE"
    KW_OBJECT_TYPE = "KW_OBJECT_TYPE"
    KW_KEYOF = "KW_KEYOF"
    KW_INFER = "KW_INFER"
    KW_IS = "KW_IS"
    KW_ASSERTS = "KW_ASSERTS"
    KW_SATISFIES = "KW_SATISFIES"

    # Delimiters
    LBRACE = "LBRACE"  # {
    RBRACE = "RBRACE"  # }
    LPAREN = "LPAREN"  # (
    RPAREN = "RPAREN"  # )
    LSQUARE = "LSQUARE"  # [
    RSQUARE = "RSQUARE"  # ]
    SEMI = "SEMI"  # ;
    COMMA = "COMMA"  # ,
    COLON = "COLON"  # :
    DOT = "DOT"  # .
    ELLIPSIS = "ELLIPSIS"  # ...
    QUESTION = "QUESTION"  # ?
    OPTIONAL_CHAIN = "OPTIONAL_CHAIN"  # ?.
    NULLISH_COALESCE = "NULLISH_COALESCE"  # ??
    AT = "AT"  # @ (decorator)
    HASH = "HASH"  # # (private field prefix)
    BACKTICK = "BACKTICK"  # `

    # Arrow
    ARROW = "ARROW"  # =>

    # Assignment Operators
    EQ = "EQ"  # =
    ADD_EQ = "ADD_EQ"  # +=
    SUB_EQ = "SUB_EQ"  # -=
    MUL_EQ = "MUL_EQ"  # *=
    DIV_EQ = "DIV_EQ"  # /=
    MOD_EQ = "MOD_EQ"  # %=
    EXP_EQ = "EXP_EQ"  # **=
    AND_EQ = "AND_EQ"  # &=
    OR_EQ = "OR_EQ"  # |=
    XOR_EQ = "XOR_EQ"  # ^=
    LSHIFT_EQ = "LSHIFT_EQ"  # <<=
    RSHIFT_EQ = "RSHIFT_EQ"  # >>=
    URSHIFT_EQ = "URSHIFT_EQ"  # >>>=
    LOGICAL_AND_EQ = "LOGICAL_AND_EQ"  # &&=
    LOGICAL_OR_EQ = "LOGICAL_OR_EQ"  # ||=
    NULLISH_EQ = "NULLISH_EQ"  # ??=

    # Comparison Operators
    EE = "EE"  # ==
    NE = "NE"  # !=
    EEE = "EEE"  # ===
    NEE = "NEE"  # !==
    LT = "LT"  # <
    GT = "GT"  # >
    LTE = "LTE"  # <=
    GTE = "GTE"  # >=

    # Arithmetic Operators
    PLUS = "PLUS"  # +
    MINUS = "MINUS"  # -
    STAR = "STAR"  # *
    SLASH = "SLASH"  # /
    PERCENT = "PERCENT"  # %
    STAR_STAR = "STAR_STAR"  # **
    PLUS_PLUS = "PLUS_PLUS"  # ++
    MINUS_MINUS = "MINUS_MINUS"  # --

    # Bitwise Operators
    BW_AND = "BW_AND"  # &
    BW_OR = "BW_OR"  # |
    BW_XOR = "BW_XOR"  # ^
    BW_NOT = "BW_NOT"  # ~
    LSHIFT = "LSHIFT"  # <<
    RSHIFT = "RSHIFT"  # >>
    URSHIFT = "URSHIFT"  # >>>

    # Logical Operators
    NOT = "NOT"  # !
    AND = "AND"  # &&
    OR = "OR"  # ||

    # JSX Tokens
    JSX_OPEN_START = "JSX_OPEN_START"  # < (in JSX context)
    JSX_CLOSE_START = "JSX_CLOSE_START"  # </
    JSX_SELF_CLOSE = "JSX_SELF_CLOSE"  # />
    JSX_TAG_END = "JSX_TAG_END"  # > (in JSX context)
    JSX_FRAG_OPEN = "JSX_FRAG_OPEN"  # <>
    JSX_FRAG_CLOSE = "JSX_FRAG_CLOSE"  # </>
    JSX_NAME = "JSX_NAME"
    JSX_TEXT = "JSX_TEXT"

    # Comments (for lexer callback)
    COMMENT = "COMMENT"
    MULTILINE_COMMENT = "MULTILINE_COMMENT"

    # Whitespace/Newline (for ASI)
    NEWLINE = "NEWLINE"
    WS = "WS"


# Token to string value mapping for TypeScript
TS_TOKEN_VALUES = {
    # Keywords - Declarations
    TsTokens.KW_VAR: "var",
    TsTokens.KW_CONST: "const",
    TsTokens.KW_FUNCTION: "function",
    TsTokens.KW_CLASS: "class",
    TsTokens.KW_INTERFACE: "interface",
    TsTokens.KW_TYPE: "type",
    TsTokens.KW_ENUM: "enum",
    TsTokens.KW_NAMESPACE: "namespace",
    TsTokens.KW_MODULE: "module",
    TsTokens.KW_DECLARE: "declare",
    # Keywords - Class/Interface
    TsTokens.KW_EXTENDS: "extends",
    TsTokens.KW_IMPLEMENTS: "implements",
    TsTokens.KW_STATIC: "static",
    TsTokens.KW_PUBLIC: "public",
    TsTokens.KW_PRIVATE: "private",
    TsTokens.KW_PROTECTED: "protected",
    TsTokens.KW_READONLY: "readonly",
    TsTokens.KW_ABSTRACT: "abstract",
    TsTokens.KW_OVERRIDE: "override",
    TsTokens.KW_CONSTRUCTOR: "constructor",
    TsTokens.KW_GET: "get",
    TsTokens.KW_SET: "set",
    # Keywords - Control Flow
    TsTokens.KW_IF: "if",
    TsTokens.KW_ELSE: "else",
    TsTokens.KW_SWITCH: "switch",
    TsTokens.KW_CASE: "case",
    TsTokens.KW_DEFAULT: "default",
    TsTokens.KW_FOR: "for",
    TsTokens.KW_WHILE: "while",
    TsTokens.KW_DO: "do",
    TsTokens.KW_BREAK: "break",
    TsTokens.KW_CONTINUE: "continue",
    TsTokens.KW_RETURN: "return",
    TsTokens.KW_THROW: "throw",
    TsTokens.KW_TRY: "try",
    TsTokens.KW_CATCH: "catch",
    TsTokens.KW_FINALLY: "finally",
    TsTokens.KW_WITH: "with",
    TsTokens.KW_DEBUGGER: "debugger",
    # Keywords - Operators
    TsTokens.KW_NEW: "new",
    TsTokens.KW_DELETE: "delete",
    TsTokens.KW_TYPEOF: "typeof",
    TsTokens.KW_VOID: "void",
    TsTokens.KW_IN: "in",
    TsTokens.KW_OF: "of",
    TsTokens.KW_INSTANCEOF: "instanceof",
    # Keywords - Async
    TsTokens.KW_ASYNC: "async",
    TsTokens.KW_AWAIT: "await",
    TsTokens.KW_YIELD: "yield",
    # Keywords - Module
    TsTokens.KW_IMPORT: "import",
    TsTokens.KW_EXPORT: "export",
    TsTokens.KW_FROM: "from",
    TsTokens.KW_AS: "as",
    # Keywords - Special
    TsTokens.KW_THIS: "this",
    TsTokens.KW_SUPER: "super",
    # Literals
    TsTokens.TRUE: "true",
    TsTokens.FALSE: "false",
    TsTokens.NULL: "null",
    TsTokens.UNDEFINED: "undefined",
    # TypeScript Type Keywords
    TsTokens.KW_ANY: "any",
    TsTokens.KW_UNKNOWN: "unknown",
    TsTokens.KW_NEVER: "never",
    TsTokens.KW_STRING_TYPE: "string",
    TsTokens.KW_NUMBER_TYPE: "number",
    TsTokens.KW_BOOLEAN_TYPE: "boolean",
    TsTokens.KW_SYMBOL_TYPE: "symbol",
    TsTokens.KW_BIGINT_TYPE: "bigint",
    TsTokens.KW_OBJECT_TYPE: "object",
    TsTokens.KW_KEYOF: "keyof",
    TsTokens.KW_INFER: "infer",
    TsTokens.KW_IS: "is",
    TsTokens.KW_ASSERTS: "asserts",
    TsTokens.KW_SATISFIES: "satisfies",
    # Delimiters
    TsTokens.LBRACE: "{",
    TsTokens.RBRACE: "}",
    TsTokens.LPAREN: "(",
    TsTokens.RPAREN: ")",
    TsTokens.LSQUARE: "[",
    TsTokens.RSQUARE: "]",
    TsTokens.SEMI: ";",
    TsTokens.COMMA: ",",
    TsTokens.COLON: ":",
    TsTokens.DOT: ".",
    TsTokens.ELLIPSIS: "...",
    TsTokens.QUESTION: "?",
    TsTokens.OPTIONAL_CHAIN: "?.",
    TsTokens.NULLISH_COALESCE: "??",
    TsTokens.AT: "@",
    TsTokens.HASH: "#",
    TsTokens.BACKTICK: "`",
    # Arrow
    TsTokens.ARROW: "=>",
    # Assignment Operators
    TsTokens.EQ: "=",
    TsTokens.ADD_EQ: "+=",
    TsTokens.SUB_EQ: "-=",
    TsTokens.MUL_EQ: "*=",
    TsTokens.DIV_EQ: "/=",
    TsTokens.MOD_EQ: "%=",
    TsTokens.EXP_EQ: "**=",
    TsTokens.AND_EQ: "&=",
    TsTokens.OR_EQ: "|=",
    TsTokens.XOR_EQ: "^=",
    TsTokens.LSHIFT_EQ: "<<=",
    TsTokens.RSHIFT_EQ: ">>=",
    TsTokens.URSHIFT_EQ: ">>>=",
    TsTokens.LOGICAL_AND_EQ: "&&=",
    TsTokens.LOGICAL_OR_EQ: "||=",
    TsTokens.NULLISH_EQ: "??=",
    # Comparison Operators
    TsTokens.EE: "==",
    TsTokens.NE: "!=",
    TsTokens.EEE: "===",
    TsTokens.NEE: "!==",
    TsTokens.LT: "<",
    TsTokens.GT: ">",
    TsTokens.LTE: "<=",
    TsTokens.GTE: ">=",
    # Arithmetic Operators
    TsTokens.PLUS: "+",
    TsTokens.MINUS: "-",
    TsTokens.STAR: "*",
    TsTokens.SLASH: "/",
    TsTokens.PERCENT: "%",
    TsTokens.STAR_STAR: "**",
    TsTokens.PLUS_PLUS: "++",
    TsTokens.MINUS_MINUS: "--",
    # Bitwise Operators
    TsTokens.BW_AND: "&",
    TsTokens.BW_OR: "|",
    TsTokens.BW_XOR: "^",
    TsTokens.BW_NOT: "~",
    TsTokens.LSHIFT: "<<",
    TsTokens.RSHIFT: ">>",
    TsTokens.URSHIFT: ">>>",
    # Logical Operators
    TsTokens.NOT: "!",
    TsTokens.AND: "&&",
    TsTokens.OR: "||",
    # JSX
    TsTokens.JSX_SELF_CLOSE: "/>",
    TsTokens.JSX_CLOSE_START: "</",
    TsTokens.JSX_FRAG_OPEN: "<>",
    TsTokens.JSX_FRAG_CLOSE: "</>",
}


class TsSymbolType(StrEnum):
    """TypeScript-specific symbol types."""

    VARIABLE = "variable"
    FUNCTION = "function"
    CLASS = "class"
    INTERFACE = "interface"
    TYPE_ALIAS = "type_alias"
    ENUM = "enum"
    ENUM_MEMBER = "enum_member"
    NAMESPACE = "namespace"
    METHOD = "method"
    PROPERTY = "property"
    PARAMETER = "parameter"
    TYPE_PARAMETER = "type_parameter"
    GETTER = "getter"
    SETTER = "setter"
    CONSTRUCTOR = "constructor"
    IMPORT = "import"
    EXPORT = "export"


class TsModifier(StrEnum):
    """TypeScript modifiers."""

    PUBLIC = "public"
    PRIVATE = "private"
    PROTECTED = "protected"
    STATIC = "static"
    READONLY = "readonly"
    ABSTRACT = "abstract"
    ASYNC = "async"
    CONST = "const"
    DECLARE = "declare"
    EXPORT = "export"
    DEFAULT = "default"
    OVERRIDE = "override"



from .exceptions import (
    GrammarError,
    LarkError,
    LexError,
    ParseError,
    UnexpectedCharacters,
    UnexpectedEOF,
    UnexpectedInput,
    UnexpectedToken,
)
from .lark import Lark
from .lexer import Token
from .tree import ParseTree, Tree
from .utils import logger
from .visitors import Discard, Transformer, Transformer_NonRecursive, Visitor, v_args

__version__: str = "1.2.2"

__all__ = (
    "GrammarError",
    "LarkError",
    "LexError",
    "ParseError",
    "UnexpectedCharacters",
    "UnexpectedEOF",
    "UnexpectedInput",
    "UnexpectedToken",
    "Lark",
    "Token",
    "ParseTree",
    "Tree",
    "logger",
    "Discard",
    "Transformer",
    "Transformer_NonRecursive",
    "Visitor",
    "v_args",
)



import re
from _io import BytesIO, StringIO
from _typeshed import Incomplete, ReadableBuffer, SupportsRead
from collections.abc import Callable, Iterable
from typing import Any, AnyStr, Generic, Literal, TypeVar, overload
from typing_extensions import Self, TypeAlias

from webencodings import Encoding

_UnicodeInputStream: TypeAlias = str | SupportsRead[str]
_BinaryInputStream: TypeAlias = bytes | SupportsRead[bytes]
_InputStream: TypeAlias = _UnicodeInputStream | _BinaryInputStream  # noqa: Y047  # used in other files
_SupportsReadT = TypeVar("_SupportsReadT", bound=SupportsRead[Any])
_SupportsReadBytesT = TypeVar("_SupportsReadBytesT", bound=SupportsRead[bytes])

spaceCharactersBytes: frozenset[bytes]
asciiLettersBytes: frozenset[bytes]
asciiUppercaseBytes: frozenset[bytes]
spacesAngleBrackets: frozenset[bytes]
invalid_unicode_no_surrogate: str
invalid_unicode_re: re.Pattern[str]
non_bmp_invalid_codepoints: set[int]
ascii_punctuation_re: re.Pattern[str]
charsUntilRegEx: dict[tuple[Iterable[str | bytes | bytearray], bool], re.Pattern[str]]

class BufferedStream(Generic[AnyStr]):
    stream: SupportsRead[AnyStr]
    buffer: list[AnyStr]
    position: list[int]
    def __init__(self, stream: SupportsRead[AnyStr]) -> None: ...
    def tell(self) -> int: ...
    def seek(self, pos: int) -> None: ...
    def read(self, bytes: int) -> AnyStr: ...

@overload
def HTMLInputStream(source: _UnicodeInputStream) -> HTMLUnicodeInputStream: ...
@overload
def HTMLInputStream(
    source: _BinaryInputStream,
    *,
    override_encoding: str | bytes | None = None,
    transport_encoding: str | bytes | None = None,
    same_origin_parent_encoding: str | bytes | None = None,
    likely_encoding: str | bytes | None = None,
    default_encoding: str = "windows-1252",
    useChardet: bool = True,
) -> HTMLBinaryInputStream: ...

class HTMLUnicodeInputStream:
    reportCharacterErrors: Callable[[str], None]
    newLines: list[int]
    charEncoding: tuple[Encoding, str]
    dataStream: Incomplete
    def __init__(self, source: _UnicodeInputStream) -> None: ...
    chunk: str
    chunkSize: int
    chunkOffset: int
    errors: list[str]
    prevNumLines: int
    prevNumCols: int
    def reset(self) -> None: ...
    @overload
    def openStream(self, source: _SupportsReadT) -> _SupportsReadT: ...
    @overload
    def openStream(self, source: str | None) -> StringIO: ...
    def position(self) -> tuple[int, int]: ...
    def char(self) -> str | None: ...
    def readChunk(self, chunkSize: int | None = None) -> bool: ...
    def characterErrorsUCS4(self, data: str) -> None: ...
    def characterErrorsUCS2(self, data: str) -> None: ...
    def charsUntil(self, characters: Iterable[str | bytes | bytearray], opposite: bool = False) -> str: ...
    def unget(self, char: str | None) -> None: ...

class HTMLBinaryInputStream(HTMLUnicodeInputStream):
    rawStream: Incomplete
    numBytesMeta: int
    numBytesChardet: int
    override_encoding: Incomplete
    transport_encoding: Incomplete
    same_origin_parent_encoding: Incomplete
    likely_encoding: Incomplete
    default_encoding: str
    charEncoding: tuple[Encoding, str]
    def __init__(
        self,
        source: _BinaryInputStream,
        override_encoding: str | bytes | None = None,
        transport_encoding: str | bytes | None = None,
        same_origin_parent_encoding: str | bytes | None = None,
        likely_encoding: str | bytes | None = None,
        default_encoding: str = "windows-1252",
        useChardet: bool = True,
    ) -> None: ...
    dataStream: Incomplete
    def reset(self) -> None: ...
    @overload  # type: ignore[override]
    def openStream(self, source: _SupportsReadBytesT) -> _SupportsReadBytesT: ...
    @overload  # type: ignore[override]
    def openStream(self, source: ReadableBuffer) -> BytesIO: ...
    def determineEncoding(self, chardet: bool = True): ...
    def changeEncoding(self, newEncoding: str | bytes | None) -> None: ...
    def detectBOM(self) -> Encoding | None: ...
    def detectEncodingMeta(self) -> Encoding | None: ...

class EncodingBytes(bytes):
    def __new__(self, value: bytes) -> Self: ...
    def __init__(self, value: bytes) -> None: ...
    def __iter__(self) -> Self: ...  # type: ignore[override]
    def __next__(self) -> bytes: ...
    def next(self) -> bytes: ...
    def previous(self) -> bytes: ...
    def setPosition(self, position: int) -> None: ...
    def getPosition(self) -> int | None: ...
    @property
    def position(self) -> int | None: ...
    @position.setter
    def position(self, position: int) -> None: ...
    def getCurrentByte(self) -> bytes: ...
    @property
    def currentByte(self) -> bytes: ...
    def skip(self, chars: bytes | bytearray | Iterable[bytes] = ...) -> bytes | None: ...
    def skipUntil(self, chars: bytes | bytearray | Iterable[bytes]) -> bytes | None: ...
    def matchBytes(self, bytes: bytes | bytearray) -> bool: ...
    def jumpTo(self, bytes: bytes | bytearray) -> Literal[True]: ...

class EncodingParser:
    data: EncodingBytes
    encoding: Encoding | None
    def __init__(self, data: bytes) -> None: ...
    def getEncoding(self) -> Encoding | None: ...
    def handleComment(self) -> bool: ...
    def handleMeta(self) -> bool: ...
    def handlePossibleStartTag(self) -> bool: ...
    def handlePossibleEndTag(self) -> bool: ...
    def handlePossibleTag(self, endTag: bool | None) -> bool: ...
    def handleOther(self) -> bool: ...
    def getAttribute(self) -> tuple[bytes, bytes] | None: ...

class ContentAttrParser:
    data: EncodingBytes
    def __init__(self, data: EncodingBytes) -> None: ...
    def parse(self) -> bytes | None: ...

def lookupEncoding(encoding: str | bytes | None) -> Encoding | None: ...



from typing import Any, TypeVar

from google.protobuf.descriptor_pool import DescriptorPool
from google.protobuf.message import Message

_MessageT = TypeVar("_MessageT", bound=Message)

class Error(Exception): ...
class ParseError(Error): ...
class SerializeToJsonError(Error): ...

def MessageToJson(
    message: Message,
    preserving_proto_field_name: bool = False,
    indent: int | None = 2,
    sort_keys: bool = False,
    use_integers_for_enums: bool = False,
    descriptor_pool: DescriptorPool | None = None,
    float_precision: int | None = None,
    ensure_ascii: bool = True,
    always_print_fields_with_no_presence: bool = False,
) -> str: ...
def MessageToDict(
    message: Message,
    always_print_fields_with_no_presence: bool = False,
    preserving_proto_field_name: bool = False,
    use_integers_for_enums: bool = False,
    descriptor_pool: DescriptorPool | None = None,
    float_precision: int | None = None,
) -> dict[str, Any]: ...
def Parse(
    text: bytes | str,
    message: _MessageT,
    ignore_unknown_fields: bool = False,
    descriptor_pool: DescriptorPool | None = None,
    max_recursion_depth: int = 100,
) -> _MessageT: ...
def ParseDict(
    js_dict: Any,
    message: _MessageT,
    ignore_unknown_fields: bool = False,
    descriptor_pool: DescriptorPool | None = None,
    max_recursion_depth: int = 100,
) -> _MessageT: ...



from .exceptions import (
    ExpiredSignatureError as ExpiredSignatureError,
    JOSEError as JOSEError,
    JWSError as JWSError,
    JWTError as JWTError,
)

__version__: str
__author__: str
__license__: str
__copyright__: str



class ServiceApiMixin:
    def create_service(
        self,
        task_template,
        name=None,
        labels=None,
        mode=None,
        update_config=None,
        networks=None,
        endpoint_config=None,
        endpoint_spec=None,
        rollback_config=None,
    ): ...
    def inspect_service(self, service, insert_defaults=None): ...
    def inspect_task(self, task): ...
    def remove_service(self, service): ...
    def services(self, filters=None, status=None): ...
    def service_logs(
        self,
        service,
        details: bool = False,
        follow: bool = False,
        stdout: bool = False,
        stderr: bool = False,
        since: int = 0,
        timestamps: bool = False,
        tail: str = "all",
        is_tty=None,
    ): ...
    def tasks(self, filters=None): ...
    def update_service(
        self,
        service,
        version,
        task_template=None,
        name=None,
        labels=None,
        mode=None,
        update_config=None,
        networks=None,
        endpoint_config=None,
        endpoint_spec=None,
        fetch_current_spec: bool = False,
        rollback_config=None,
    ): ...



from wsproto.frame_protocol import CloseReason

class SimpleWebsocketError(RuntimeError): ...

class ConnectionError(SimpleWebsocketError):
    status_code: int | None
    def __init__(self, status_code: int | None = None) -> None: ...

class ConnectionClosed(SimpleWebsocketError):
    reason: CloseReason
    message: str | None
    def __init__(self, reason: CloseReason = ..., message: str | None = None) -> None: ...



from typing import ClassVar

from braintree.attribute_getter import AttributeGetter

class PackageDetails(AttributeGetter):
    detail_list: ClassVar[list[str]]
    def __init__(self, attributes) -> None: ...



from _typeshed import Incomplete
from typing import Final

__version__: Final[str]

class CallableValue:
    func: Incomplete
    args: Incomplete
    kw: Incomplete
    def __init__(self, func, *args, **kw) -> None: ...
    def __call__(self): ...

class AttrMapValue:
    validate: Incomplete
    desc: Incomplete
    def __init__(self, validate=None, desc=None, initial=None, advancedUsage: int = 0, **kw) -> None: ...
    def __getattr__(self, name): ...

class AttrMap(dict[str, AttrMapValue]):
    def __init__(self, BASE=None, UNWANTED=[], **kw) -> None: ...
    def remove(self, unwanted) -> None: ...
    def clone(self, UNWANTED=[], **kw): ...

def validateSetattr(obj, name, value) -> None: ...
def hook__setattr__(obj): ...
def addProxyAttribute(src, name, validate=None, desc=None, initial=None, dst=None) -> None: ...



import logging
from typing import Iterable

logger = logging.getLogger('interegular')
logger.addHandler(logging.StreamHandler())
logger.setLevel(logging.CRITICAL)


def soft_repr(s: str):
    """
    joins together the repr of each char in the string, while throwing away the repr
    This for example turns `"'\""` into `'"` instead of `\'"` like the normal repr
    would do.
    """
    return ''.join(repr(c)[1:-1] for c in str(s))



from _typeshed import Unused
from typing import Literal, TypedDict, type_check_only

import gdb

from .varref import VariableReference, _ValueFormat, _VariableReferenceDescriptor

class EvaluateResult(VariableReference):
    def __init__(self, value: gdb.Value) -> None: ...

@type_check_only
class _VariablesResult(TypedDict):
    variables: list[_VariableReferenceDescriptor]

def eval_request(
    *,
    expression: str,
    frameId: int | None = None,
    context: Literal["watch", "variables", "hover", "repl"] = "variables",
    format: _ValueFormat | None = None,
    **args: Unused,
) -> _VariableReferenceDescriptor: ...
def variables(
    *, variablesReference: int, start: int = 0, count: int = 0, format: _ValueFormat | None = None, **args: Unused
) -> _VariablesResult: ...
def set_expression(
    *, expression: str, value: str, frameId: int | None = None, format: _ValueFormat | None = None, **args: Unused
) -> _VariableReferenceDescriptor: ...
def set_variable(
    *, variablesReference: int, name: str, value: str, format: _ValueFormat | None = None, **args: Unused
) -> _VariableReferenceDescriptor: ...



from dataclasses import dataclass

from seaborn._marks.base import MappableBool, MappableColor, MappableFloat, MappableStyle, Mark, document_properties

class BarBase(Mark): ...

@document_properties
@dataclass
class Bar(BarBase):
    color: MappableColor = ...
    alpha: MappableFloat = ...
    fill: MappableBool = ...
    edgecolor: MappableColor = ...
    edgealpha: MappableFloat = ...
    edgewidth: MappableFloat = ...
    edgestyle: MappableStyle = ...
    width: MappableFloat = ...
    baseline: MappableFloat = ...

@document_properties
@dataclass
class Bars(BarBase):
    color: MappableColor = ...
    alpha: MappableFloat = ...
    fill: MappableBool = ...
    edgecolor: MappableColor = ...
    edgealpha: MappableFloat = ...
    edgewidth: MappableFloat = ...
    edgestyle: MappableStyle = ...
    width: MappableFloat = ...
    baseline: MappableFloat = ...



from reportlab.graphics.samples.excelcolors import *
from reportlab.graphics.shapes import Drawing, _DrawingEditorMixin

class ScatterLines(_DrawingEditorMixin, Drawing):
    def __init__(self, width: int = 200, height: int = 150, *args, **kw) -> None: ...



from typing import Any

from .._typing import SupportsGeoInterface
from .base import BaseGeometry
from .polygon import Polygon

def box(minx: float, miny: float, maxx: float, maxy: float, ccw: bool = True) -> Polygon: ...
def shape(context: dict[str, Any] | SupportsGeoInterface) -> BaseGeometry: ...
def mapping(ob: SupportsGeoInterface) -> dict[str, Any]: ...



from _typeshed import Incomplete, Unused
from collections import defaultdict
from logging import Logger
from typing import Final

from .annotations import AnnotationDict
from .encryption import StandardSecurityHandler
from .enums import OutputIntentSubType, PageLabelStyle, PDFResourceType
from .fpdf import FPDF
from .image_datastructures import RasterImageInfo
from .line_break import TotalPagesSubstitutionFragment
from .syntax import Name, PDFArray, PDFContentStream, PDFObject, PDFString

LOGGER: Logger
ZOOM_CONFIGS: Final[dict[str, tuple[str, ...]]]

class ContentWithoutID:
    def serialize(self, _security_handler: StandardSecurityHandler | None = None) -> str | None: ...

class PDFHeader(ContentWithoutID):
    pdf_version: str
    def __init__(self, pdf_version: str) -> None: ...
    def serialize(self, _security_handler: StandardSecurityHandler | None = None) -> str: ...

class PDFFont(PDFObject):
    type: Name
    subtype: Name
    base_font: Name
    encoding: Name | None
    d_w: Incomplete | None
    w: Incomplete | None
    descendant_fonts: Incomplete | None
    to_unicode: Incomplete | None
    c_i_d_system_info: Incomplete | None
    font_descriptor: Incomplete | None
    c_i_d_to_g_i_d_map: Incomplete | None
    def __init__(self, subtype: str, base_font: str, encoding: str | None = None, d_w=None, w=None) -> None: ...

class CIDSystemInfo(PDFObject):
    registry: PDFString
    ordering: PDFString
    supplement: int

class PDFInfo(PDFObject):
    title: str | None
    subject: str | None
    author: str | None
    keywords: str | None
    creator: str | None
    producer: str | None
    creation_date: Incomplete
    def __init__(
        self,
        title: str | None,
        subject: str | None,
        author: str | None,
        keywords: str | None,
        creator: str | None,
        producer: str | None,
        creation_date,
    ) -> None: ...

class AcroForm:
    fields: Incomplete
    sig_flags: Incomplete
    def __init__(self, fields, sig_flags) -> None: ...
    def serialize(self) -> str: ...

class PDFCatalog(PDFObject):
    type: Name
    lang: str | None
    page_layout: Incomplete | None
    page_mode: Incomplete | None
    viewer_preferences: Incomplete | None
    pages: Incomplete | None
    acro_form: Incomplete | None
    open_action: Incomplete | None
    mark_info: Incomplete | None
    metadata: Incomplete | None
    names: Incomplete | None
    outlines: Incomplete | None
    output_intents: Incomplete | None
    struct_tree_root: Incomplete | None
    def __init__(self, lang: str | None = None, page_layout=None, page_mode=None, viewer_preferences=None) -> None: ...

class PDFResources(PDFObject):
    proc_set: Incomplete
    font: Incomplete
    x_object: Incomplete
    ext_g_state: Incomplete
    shading: Incomplete
    pattern: Incomplete
    def __init__(self, proc_set, font, x_object, ext_g_state, shading, pattern) -> None: ...

class PDFFontStream(PDFContentStream):
    length1: int
    def __init__(self, contents: bytes) -> None: ...

class PDFXmpMetadata(PDFContentStream):
    type: Name
    subtype: Name
    def __init__(self, contents: bytes) -> None: ...

class PDFXObject(PDFContentStream):
    __slots__ = (
        "_id",
        "_contents",
        "filter",
        "length",
        "type",
        "subtype",
        "width",
        "height",
        "color_space",
        "bits_per_component",
        "decode",
        "decode_parms",
        "s_mask",
    )
    type: Name
    subtype: Name
    width: Incomplete
    height: Incomplete
    color_space: Incomplete
    bits_per_component: Incomplete
    filter: Name
    decode: Incomplete | None
    decode_parms: Incomplete | None
    s_mask: Incomplete | None
    def __init__(
        self,
        contents,
        subtype: str,
        width,
        height,
        color_space,
        bits_per_component,
        img_filter: str | None = None,
        decode=None,
        decode_parms=None,
    ) -> None: ...

class PDFICCProfile(PDFContentStream):
    __slots__ = ("_id", "_contents", "filter", "length", "n", "alternate")
    n: Incomplete
    alternate: Name
    def __init__(self, contents: bytes, n, alternate: str) -> None: ...

class PDFPageLabel:
    __slots__ = ("_style", "_prefix", "st")
    st: int
    def __init__(self, label_style: PageLabelStyle, label_prefix: str, label_start: int) -> None: ...
    @property
    def s(self) -> Name: ...
    @property
    def p(self) -> PDFString: ...
    def serialize(self) -> dict[str, str]: ...
    def get_style(self) -> PageLabelStyle: ...
    def get_prefix(self) -> str: ...
    def get_start(self) -> int: ...

class PDFPage(PDFObject):
    __slots__ = (
        "_id",
        "type",
        "contents",
        "dur",
        "trans",
        "annots",
        "group",
        "media_box",
        "struct_parents",
        "resources",
        "parent",
        "_index",
        "_width_pt",
        "_height_pt",
        "_page_label",
        "_text_substitution_fragments",
    )
    type: Name
    contents: Incomplete
    dur: Incomplete | None
    trans: Incomplete
    annots: PDFArray[AnnotationDict]
    group: Incomplete | None
    media_box: Incomplete | None
    struct_parents: Incomplete | None
    resources: Incomplete | None
    parent: Incomplete | None
    def __init__(self, duration: Incomplete | None, transition, contents, index) -> None: ...
    def index(self) -> int: ...
    def set_index(self, i: int) -> None: ...
    def dimensions(self) -> tuple[float | None, float | None]: ...
    def set_dimensions(self, width_pt: float | None, height_pt: float | None) -> None: ...
    def set_page_label(self, previous_page_label: PDFPageLabel, page_label: PDFPageLabel) -> None: ...
    def get_page_label(self) -> PDFPageLabel: ...
    def get_label(self) -> str: ...
    def get_text_substitutions(self) -> list[TotalPagesSubstitutionFragment]: ...
    def add_text_substitution(self, fragment: TotalPagesSubstitutionFragment) -> None: ...

class PDFPagesRoot(PDFObject):
    type: Name
    count: Incomplete
    media_box: Incomplete
    kids: Incomplete | None
    def __init__(self, count, media_box) -> None: ...

class PDFExtGState(PDFObject):
    def __init__(self, dict_as_str) -> None: ...
    def serialize(self, obj_dict: Unused = None, _security_handler: StandardSecurityHandler | None = None) -> str: ...

class PDFXrefAndTrailer(ContentWithoutID):
    output_builder: Incomplete
    count: int
    catalog_obj: Incomplete | None
    info_obj: Incomplete | None
    def __init__(self, output_builder) -> None: ...
    def serialize(self, _security_handler: StandardSecurityHandler | None = None) -> str: ...

class OutputIntentDictionary:
    __slots__ = ("type", "s", "output_condition_identifier", "output_condition", "registry_name", "dest_output_profile", "info")
    type: Name
    s: Name
    output_condition_identifier: PDFString | None
    output_condition: PDFString | None
    registry_name: PDFString | None
    dest_output_profile: Incomplete | None
    info: PDFString | None

    def __init__(
        self,
        subtype: OutputIntentSubType | str,
        output_condition_identifier: str,
        output_condition: str | None = None,
        registry_name: str | None = None,
        dest_output_profile: PDFICCProfile | None = None,
        info: str | None = None,
    ) -> None: ...
    def serialize(self, _security_handler: StandardSecurityHandler | None = None, _obj_id=None): ...

class ResourceCatalog:
    resources: defaultdict[PDFResourceType, dict[Incomplete, Incomplete]]
    resources_per_page: defaultdict[tuple[int, PDFResourceType], set[Incomplete]]

    def add(self, resource_type: PDFResourceType, resource, page_number: int) -> Incomplete | None: ...
    def get_items(self, resource_type: PDFResourceType): ...
    def get_resources_per_page(self, page_number: int, resource_type: PDFResourceType): ...
    def get_used_resources(self, resource_type: PDFResourceType) -> set[Incomplete]: ...

class OutputProducer:
    fpdf: FPDF
    pdf_objs: list[Incomplete]
    obj_id: int
    offsets: dict[Incomplete, Incomplete]
    trace_labels_per_obj_id: dict[Incomplete, Incomplete]
    sections_size_per_trace_label: defaultdict[Incomplete, int]
    buffer: bytearray
    def __init__(self, fpdf: FPDF) -> None: ...
    def bufferize(self) -> bytearray: ...

def stream_content_for_raster_image(
    info: RasterImageInfo,
    x: float,
    y: float,
    w: float,
    h: float,
    keep_aspect_ratio: bool = False,
    scale: float = 1,
    pdf_height_to_flip: float | None = None,
) -> str: ...



from _typeshed import Incomplete
from abc import abstractmethod
from typing import Any

from dateparser.conf import Settings
from dateparser.parser import _parser

class CalendarBase:
    parser: Any
    source: Any
    def __init__(self, source) -> None: ...
    def get_date(self): ...

class non_gregorian_parser(_parser):
    calendar_converter: Any
    default_year: Any
    default_month: Any
    default_day: Any
    non_gregorian_date_cls: Any
    @classmethod
    def to_latin(cls, source): ...
    @abstractmethod
    def handle_two_digit_year(self, year: int) -> int: ...
    @classmethod
    def parse(cls, datestring: str, settings: Settings) -> tuple[Incomplete, Incomplete]: ...  # type: ignore[override]



from _typeshed import Incomplete

from reportlab.graphics.barcode.common import Barcode

class USPS_4State(Barcode):
    tops: Incomplete
    bottoms: Incomplete
    dimensions: Incomplete
    def __init__(self, value: str = "01234567094987654321", routing: str = "", **kwd) -> None: ...
    @staticmethod
    def scale(kind, D, s): ...
    @property
    def tracking(self): ...
    @tracking.setter
    def tracking(self, tracking) -> None: ...
    @property
    def routing(self): ...
    @routing.setter
    def routing(self, routing) -> None: ...
    @property
    def widthSize(self): ...
    @widthSize.setter
    def widthSize(self, value) -> None: ...
    @property
    def heightSize(self): ...
    @heightSize.setter
    def heightSize(self, value) -> None: ...
    @property
    def fontSize(self): ...
    @fontSize.setter
    def fontSize(self, value) -> None: ...
    @property
    def humanReadable(self): ...
    @humanReadable.setter
    def humanReadable(self, value) -> None: ...
    @property
    def binary(self): ...
    @property
    def codewords(self): ...
    @property
    def table1(self): ...
    @property
    def table2(self): ...
    @property
    def characters(self): ...
    @property
    def barcodes(self): ...
    table4: Incomplete
    @property
    def horizontalClearZone(self): ...
    @property
    def verticalClearZone(self): ...
    @property
    def barWidth(self): ...
    @barWidth.setter
    def barWidth(self, value) -> None: ...
    @property
    def pitch(self): ...
    @pitch.setter
    def pitch(self, value) -> None: ...
    @property
    def barHeight(self): ...
    @barHeight.setter
    def barHeight(self, value) -> None: ...
    @property
    def widthScale(self): ...
    @property
    def heightScale(self): ...
    @property
    def width(self): ...
    @width.setter
    def width(self, v) -> None: ...
    @property
    def height(self): ...
    @height.setter
    def height(self, v) -> None: ...
    def computeSize(self) -> None: ...
    def wrap(self, aW, aH): ...
    def draw(self) -> None: ...
    @property
    def value(self): ...
    @value.setter
    def value(self, value) -> None: ...
    def drawHumanReadable(self) -> None: ...
    def annotate(self, x, y, text, fontName, fontSize, anchor: str = "middle") -> None: ...

__all__ = ("USPS_4State",)



import abc
from _typeshed import Incomplete
from typing import Any, ClassVar
from typing_extensions import Self

from passlib.ifc import PasswordHash
from passlib.utils.binary import BASE64_CHARS, HASH64_CHARS, LOWER_HEX_CHARS, PADDED_BASE64_CHARS, UPPER_HEX_CHARS

H64_CHARS = HASH64_CHARS
B64_CHARS = BASE64_CHARS
PADDED_B64_CHARS = PADDED_BASE64_CHARS
UC_HEX_CHARS = UPPER_HEX_CHARS
LC_HEX_CHARS = LOWER_HEX_CHARS

def parse_mc2(hash, prefix, sep="$", handler=None): ...
def parse_mc3(hash, prefix, sep="$", rounds_base: int = 10, default_rounds=None, handler=None): ...
def render_mc2(ident, salt, checksum, sep="$"): ...
def render_mc3(ident, rounds, salt, checksum, sep="$", rounds_base: int = 10): ...

class MinimalHandler(PasswordHash, metaclass=abc.ABCMeta):
    @classmethod
    def using(cls, relaxed: bool = False) -> type[Self]: ...  # type: ignore[override]

class TruncateMixin(MinimalHandler, metaclass=abc.ABCMeta):
    truncate_error: ClassVar[bool]
    truncate_verify_reject: ClassVar[bool]
    @classmethod
    def using(cls, truncate_error: object = None, **kwds: Any) -> type[Self]: ...  # type: ignore[override]

class GenericHandler(MinimalHandler):
    setting_kwds: ClassVar[tuple[str, ...]]
    context_kwds: ClassVar[tuple[str, ...]]
    ident: ClassVar[str | None]
    checksum_size: ClassVar[int | None]
    checksum_chars: ClassVar[str | None]
    checksum: str | None
    use_defaults: bool
    def __init__(self, checksum: str | None = None, use_defaults: bool = False) -> None: ...
    @classmethod
    def identify(cls, hash: str | bytes) -> bool: ...
    @classmethod
    def from_string(cls, hash: str | bytes, **context: Any) -> Self: ...  # context parameters are passed to constructor
    def to_string(self) -> str: ...
    @classmethod
    def hash(cls, secret: str | bytes, **kwds: Any) -> str: ...
    @classmethod
    def verify(cls, secret: str | bytes, hash: str | bytes, **context: Any) -> bool: ...
    @classmethod
    def genconfig(cls, **kwds: Any) -> str: ...
    @classmethod
    def genhash(cls, secret: str | bytes, config: str, **context: Any) -> str: ...
    @classmethod
    def needs_update(cls, hash: str | bytes, secret: str | bytes | None = None, **kwds: Any) -> bool: ...
    @classmethod
    def parsehash(cls, hash: str | bytes, checksum: bool = True, sanitize: bool = False) -> dict[str, Any]: ...
    @classmethod
    def bitsize(cls, **kwds: Any) -> dict[str, Any]: ...

class StaticHandler(GenericHandler):
    setting_kwds: ClassVar[tuple[str, ...]]

class HasEncodingContext(GenericHandler):
    default_encoding: ClassVar[str]
    encoding: str
    def __init__(self, encoding: str | None = None, **kwds) -> None: ...

class HasUserContext(GenericHandler):
    user: Incomplete | None
    def __init__(self, user=None, **kwds) -> None: ...
    @classmethod
    def hash(cls, secret, user=None, **context): ...
    @classmethod
    def verify(cls, secret, hash, user=None, **context): ...
    @classmethod
    def genhash(cls, secret, config, user=None, **context): ...

class HasRawChecksum(GenericHandler): ...

class HasManyIdents(GenericHandler):
    default_ident: ClassVar[str | None]
    ident_values: ClassVar[tuple[str, ...] | None]
    ident_aliases: ClassVar[dict[str, str] | None]
    ident: str  # type: ignore[misc]
    @classmethod
    def using(cls, default_ident=None, ident=None, **kwds): ...  # type: ignore[override]
    def __init__(self, ident=None, **kwds) -> None: ...

class HasSalt(GenericHandler):
    min_salt_size: ClassVar[int]
    max_salt_size: ClassVar[int | None]
    salt_chars: ClassVar[str | None]
    default_salt_size: ClassVar[int | None]
    default_salt_chars: ClassVar[str | None]
    salt: str | bytes | None
    @classmethod
    def using(  # type: ignore[override]
        cls, default_salt_size: int | None = None, salt_size: int | None = None, salt: str | bytes | None = None, **kwds
    ): ...
    def __init__(self, salt: str | bytes | None = None, **kwds) -> None: ...
    @classmethod
    def bitsize(cls, salt_size: int | None = None, **kwds): ...

class HasRawSalt(HasSalt):
    salt_chars: ClassVar[bytes]  # type: ignore[assignment]

class HasRounds(GenericHandler):
    min_rounds: ClassVar[int]
    max_rounds: ClassVar[int | None]
    rounds_cost: ClassVar[str]
    using_rounds_kwds: ClassVar[tuple[str, ...]]
    min_desired_rounds: ClassVar[int | None]
    max_desired_rounds: ClassVar[int | None]
    default_rounds: ClassVar[int | None]
    vary_rounds: ClassVar[Incomplete | None]
    rounds: int
    @classmethod
    def using(  # type: ignore[override]
        cls,
        min_desired_rounds=None,
        max_desired_rounds=None,
        default_rounds=None,
        vary_rounds=None,
        min_rounds=None,
        max_rounds=None,
        rounds=None,
        **kwds,
    ): ...
    def __init__(self, rounds=None, **kwds) -> None: ...
    @classmethod
    def bitsize(cls, rounds=None, vary_rounds: float = 0.1, **kwds): ...

class ParallelismMixin(GenericHandler):
    parallelism: int
    @classmethod
    def using(cls, parallelism=None, **kwds): ...  # type: ignore[override]
    def __init__(self, parallelism=None, **kwds) -> None: ...

class BackendMixin(PasswordHash, metaclass=abc.ABCMeta):
    backends: ClassVar[tuple[str, ...] | None]
    @classmethod
    def get_backend(cls): ...
    @classmethod
    def has_backend(cls, name: str = "any") -> bool: ...
    @classmethod
    def set_backend(cls, name: str = "any", dryrun: bool = False): ...

class SubclassBackendMixin(BackendMixin, metaclass=abc.ABCMeta): ...
class HasManyBackends(BackendMixin, GenericHandler): ...

class PrefixWrapper:
    name: Any
    prefix: Any
    orig_prefix: Any
    __doc__: Any
    def __init__(self, name, wrapped, prefix="", orig_prefix="", lazy: bool = False, doc=None, ident=None) -> None: ...
    @property
    def wrapped(self): ...
    @property
    def ident(self): ...
    @property
    def ident_values(self): ...
    def __dir__(self): ...
    def __getattr__(self, attr: str): ...
    def __setattr__(self, attr: str, value) -> None: ...
    def using(self, **kwds): ...
    def needs_update(self, hash, **kwds): ...
    def identify(self, hash): ...
    def genconfig(self, **kwds): ...
    def genhash(self, secret, config, **kwds): ...
    def encrypt(self, secret, **kwds): ...
    def hash(self, secret, **kwds): ...
    def verify(self, secret, hash, **kwds): ...

__all__ = [
    # helpers for implementing MCF handlers
    "parse_mc2",
    "parse_mc3",
    "render_mc2",
    "render_mc3",
    # framework for implementing handlers
    "GenericHandler",
    "StaticHandler",
    "HasUserContext",
    "HasRawChecksum",
    "HasManyIdents",
    "HasSalt",
    "HasRawSalt",
    "HasRounds",
    "HasManyBackends",
    # other helpers
    "PrefixWrapper",
]



from typing_extensions import Self

class XRayTracedConn:
    def __init__(self, conn, meta={}) -> None: ...
    def cursor(self, *args, **kwargs) -> XRayTracedCursor: ...

class XRayTracedCursor:
    def __init__(self, cursor, meta={}) -> None: ...
    def __enter__(self) -> Self: ...
    def execute(self, query, *args, **kwargs): ...
    def executemany(self, query, *args, **kwargs): ...
    def callproc(self, proc, args): ...

def add_sql_meta(meta) -> None: ...



from dataclasses import dataclass

from seaborn._marks.base import MappableBool, MappableColor, MappableFloat, MappableStyle, Mark, document_properties

class AreaBase: ...

@document_properties
@dataclass
class Area(AreaBase, Mark):
    color: MappableColor = ...
    alpha: MappableFloat = ...
    fill: MappableBool = ...
    edgecolor: MappableColor = ...
    edgealpha: MappableFloat = ...
    edgewidth: MappableFloat = ...
    edgestyle: MappableStyle = ...
    baseline: MappableFloat = ...

@document_properties
@dataclass
class Band(AreaBase, Mark):
    color: MappableColor = ...
    alpha: MappableFloat = ...
    fill: MappableBool = ...
    edgecolor: MappableColor = ...
    edgealpha: MappableFloat = ...
    edgewidth: MappableFloat = ...
    edgestyle: MappableFloat = ...



import collections
import types
from collections.abc import Callable, Collection, Iterable, Mapping
from typing import Any, NoReturn
from typing_extensions import TypeAlias

from yt_dlp.extractor.common import InfoExtractor
from yt_dlp.utils._utils import function_with_repr

from .utils import ExtractorError

def js_number_to_string(val: float, radix: int = 10) -> str: ...

class JS_Undefined: ...

class JS_Break(ExtractorError):
    def __init__(self) -> None: ...

class JS_Continue(ExtractorError):
    def __init__(self) -> None: ...

class JS_Throw(ExtractorError):
    error: BaseException
    def __init__(self, e: BaseException) -> None: ...

class LocalNameSpace(collections.ChainMap[str, Any]):
    def __setitem__(self, key: str, value: Any) -> None: ...
    def __delitem__(self, key: str) -> NoReturn: ...
    def set_local(self, key: str, value: Any) -> None: ...
    def get_local(self, key: str) -> Any: ...

class Debugger:
    ENABLED: bool
    @staticmethod
    def write(*args: str, level: int = 100) -> None: ...
    @classmethod
    # Callable[[Debugger, str, Any, int, ...], tuple[Any, bool]] but it also accepts *args, **kwargs.
    def wrap_interpreter(cls, f: Callable[..., tuple[Any, bool]]) -> Callable[..., tuple[Any, bool]]: ...

_BuildFunctionReturnType: TypeAlias = Callable[[Collection[Any], Mapping[str, Any], int], Any | None]

class JSInterpreter:
    def __init__(self, code: str, objects: Mapping[str, Any] | None = None) -> None: ...

    class Exception(ExtractorError):
        def __init__(
            self,
            msg: str,
            expr: str | None = None,
            tb: types.TracebackType | None = None,
            expected: bool = False,
            cause: Exception | str | None = None,
            video_id: str | None = None,
            ie: InfoExtractor | None = None,
        ) -> None: ...

    # After wrapping, *args and **kwargs are added but do nothing for this method.
    def interpret_statement(
        self, stmt: str, local_vars: Mapping[str, Any], allow_recursion: int, *args: Any, **kwargs: Any
    ) -> tuple[Any, bool]: ...
    def interpret_expression(self, expr: str, local_vars: Mapping[str, Any], allow_recursion: int) -> Any: ...
    def extract_object(self, objname: str, *global_stack: Iterable[dict[str, Any]]) -> Any: ...
    def extract_function_code(self, funcname: str) -> tuple[list[str], tuple[str, str]]: ...
    def extract_function(self, funcname: str, *global_stack: Iterable[dict[str, Any]]) -> function_with_repr[Any]: ...
    def extract_function_from_code(
        self, argnames: Collection[str], code: str, *global_stack: Iterable[dict[str, Any]]
    ) -> _BuildFunctionReturnType: ...
    # args are passed to the extracted function.
    def call_function(self, funcname: str, *args: Any) -> function_with_repr[Any]: ...
    def build_function(
        self, argnames: Collection[str], code: str, *global_stack: Iterable[dict[str, Any]]
    ) -> _BuildFunctionReturnType: ...



from _typeshed import Incomplete
from logging import Logger

from oauthlib.common import Request

from ..tokens import TokenBase
from .base import GrantTypeBase

log: Logger

class ImplicitGrant(GrantTypeBase):
    response_types: list[str]
    grant_allows_refresh_token: bool
    def create_authorization_response(
        self, request: Request, token_handler: TokenBase
    ) -> tuple[dict[str, str], str | None, int]: ...
    def create_token_response(self, request: Request, token_handler: TokenBase) -> tuple[dict[str, str], str | None, int]: ...
    def validate_authorization_request(self, request: Request) -> tuple[Incomplete, dict[str, Incomplete]]: ...
    def validate_token_request(self, request: Request) -> tuple[Incomplete, dict[str, Incomplete]]: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import tensorflow.core.framework.attr_value_pb2
import tensorflow.core.framework.node_def_pb2
import tensorflow.core.framework.op_def_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class FunctionDefLibrary(google.protobuf.message.Message):
    """A library is a set of named functions."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FUNCTION_FIELD_NUMBER: builtins.int
    GRADIENT_FIELD_NUMBER: builtins.int
    REGISTERED_GRADIENTS_FIELD_NUMBER: builtins.int
    @property
    def function(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___FunctionDef]: ...
    @property
    def gradient(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___GradientDef]: ...
    @property
    def registered_gradients(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___RegisteredGradient]: ...
    def __init__(
        self,
        *,
        function: collections.abc.Iterable[global___FunctionDef] | None = ...,
        gradient: collections.abc.Iterable[global___GradientDef] | None = ...,
        registered_gradients: collections.abc.Iterable[global___RegisteredGradient] | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "function", b"function", "gradient", b"gradient", "registered_gradients", b"registered_gradients"
        ],
    ) -> None: ...

global___FunctionDefLibrary = FunctionDefLibrary

@typing.final
class FunctionDef(google.protobuf.message.Message):
    """A function can be instantiated when the runtime can bind every attr
    with a value. When a GraphDef has a call to a function, it must
    have binding for every attr defined in the signature.

    TODO(zhifengc):
      * device spec, etc.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class AttrEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> tensorflow.core.framework.attr_value_pb2.AttrValue: ...
        def __init__(
            self, *, key: builtins.str | None = ..., value: tensorflow.core.framework.attr_value_pb2.AttrValue | None = ...
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    @typing.final
    class ArgAttrs(google.protobuf.message.Message):
        """Attributes for function arguments. These attributes are the same set of
        valid attributes as to _Arg nodes.
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        @typing.final
        class AttrEntry(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            KEY_FIELD_NUMBER: builtins.int
            VALUE_FIELD_NUMBER: builtins.int
            key: builtins.str
            @property
            def value(self) -> tensorflow.core.framework.attr_value_pb2.AttrValue: ...
            def __init__(
                self, *, key: builtins.str | None = ..., value: tensorflow.core.framework.attr_value_pb2.AttrValue | None = ...
            ) -> None: ...
            def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
            def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

        ATTR_FIELD_NUMBER: builtins.int
        @property
        def attr(
            self,
        ) -> google.protobuf.internal.containers.MessageMap[builtins.str, tensorflow.core.framework.attr_value_pb2.AttrValue]: ...
        def __init__(
            self, *, attr: collections.abc.Mapping[builtins.str, tensorflow.core.framework.attr_value_pb2.AttrValue] | None = ...
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["attr", b"attr"]) -> None: ...

    @typing.final
    class ArgAttrEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.int
        @property
        def value(self) -> global___FunctionDef.ArgAttrs: ...
        def __init__(self, *, key: builtins.int | None = ..., value: global___FunctionDef.ArgAttrs | None = ...) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    @typing.final
    class ResourceArgUniqueIdEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.int
        value: builtins.int
        def __init__(self, *, key: builtins.int | None = ..., value: builtins.int | None = ...) -> None: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    @typing.final
    class RetEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(self, *, key: builtins.str | None = ..., value: builtins.str | None = ...) -> None: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    @typing.final
    class ControlRetEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(self, *, key: builtins.str | None = ..., value: builtins.str | None = ...) -> None: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    SIGNATURE_FIELD_NUMBER: builtins.int
    ATTR_FIELD_NUMBER: builtins.int
    ARG_ATTR_FIELD_NUMBER: builtins.int
    RESOURCE_ARG_UNIQUE_ID_FIELD_NUMBER: builtins.int
    NODE_DEF_FIELD_NUMBER: builtins.int
    RET_FIELD_NUMBER: builtins.int
    CONTROL_RET_FIELD_NUMBER: builtins.int
    @property
    def signature(self) -> tensorflow.core.framework.op_def_pb2.OpDef:
        """The definition of the function's name, arguments, return values,
        attrs etc.
        """

    @property
    def attr(
        self,
    ) -> google.protobuf.internal.containers.MessageMap[builtins.str, tensorflow.core.framework.attr_value_pb2.AttrValue]:
        """Attributes specific to this function definition."""

    @property
    def arg_attr(self) -> google.protobuf.internal.containers.MessageMap[builtins.int, global___FunctionDef.ArgAttrs]: ...
    @property
    def resource_arg_unique_id(self) -> google.protobuf.internal.containers.ScalarMap[builtins.int, builtins.int]:
        """Unique IDs for each resource argument, used to track aliasing resources. If
        Argument A and Argument B alias each other, then
        resource_arg_unique_ids[A.index] == resource_arg_unique_ids[B.index].

        If this field is empty, none of the arguments could alias; otherwise, every
        resource argument should have an entry in this field.

        When instantiated, the unique IDs will be attached to the _Arg nodes'
        "_resource_arg_unique_id" attribute.
        """

    @property
    def node_def(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[tensorflow.core.framework.node_def_pb2.NodeDef]:
        """The body of the function.  Unlike the NodeDefs in a GraphDef, attrs
        may have values of type `placeholder` and the `input` field uses
        the "output" format above.

        By convention, "op" in node_def is resolved by consulting with a
        user-defined library first. If not resolved, "func" is assumed to
        be a builtin op.
        """

    @property
    def ret(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """A mapping from the output arg names from `signature` to the
        outputs from `node_def` that should be returned by the function.
        """

    @property
    def control_ret(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """A mapping from control output names from `signature` to node names in
        `node_def` which should be control outputs of this function.
        """

    def __init__(
        self,
        *,
        signature: tensorflow.core.framework.op_def_pb2.OpDef | None = ...,
        attr: collections.abc.Mapping[builtins.str, tensorflow.core.framework.attr_value_pb2.AttrValue] | None = ...,
        arg_attr: collections.abc.Mapping[builtins.int, global___FunctionDef.ArgAttrs] | None = ...,
        resource_arg_unique_id: collections.abc.Mapping[builtins.int, builtins.int] | None = ...,
        node_def: collections.abc.Iterable[tensorflow.core.framework.node_def_pb2.NodeDef] | None = ...,
        ret: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        control_ret: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["signature", b"signature"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "arg_attr",
            b"arg_attr",
            "attr",
            b"attr",
            "control_ret",
            b"control_ret",
            "node_def",
            b"node_def",
            "resource_arg_unique_id",
            b"resource_arg_unique_id",
            "ret",
            b"ret",
            "signature",
            b"signature",
        ],
    ) -> None: ...

global___FunctionDef = FunctionDef

@typing.final
class GradientDef(google.protobuf.message.Message):
    """GradientDef defines the gradient function of a function defined in
    a function library.

    A gradient function g (specified by gradient_func) for a function f
    (specified by function_name) must follow the following:

    The function 'f' must be a numerical function which takes N inputs
    and produces M outputs. Its gradient function 'g', which is a
    function taking N + M inputs and produces N outputs.

    I.e. if we have
       (y1, y2, ..., y_M) = f(x1, x2, ..., x_N),
    then, g is
       (dL/dx1, dL/dx2, ..., dL/dx_N) = g(x1, x2, ..., x_N,
                                         dL/dy1, dL/dy2, ..., dL/dy_M),
    where L is a scalar-value function of (x1, x2, ..., xN) (e.g., the
    loss function). dL/dx_i is the partial derivative of L with respect
    to x_i.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FUNCTION_NAME_FIELD_NUMBER: builtins.int
    GRADIENT_FUNC_FIELD_NUMBER: builtins.int
    function_name: builtins.str
    """The function name."""
    gradient_func: builtins.str
    """The gradient function's name."""
    def __init__(self, *, function_name: builtins.str | None = ..., gradient_func: builtins.str | None = ...) -> None: ...
    def ClearField(
        self, field_name: typing.Literal["function_name", b"function_name", "gradient_func", b"gradient_func"]
    ) -> None: ...

global___GradientDef = GradientDef

@typing.final
class RegisteredGradient(google.protobuf.message.Message):
    """RegisteredGradient stores a gradient function that is registered in the
    gradients library and used in the ops of a function in the function library.
    Unlike GradientDef, these gradients are identified by op type, and not
    directly linked to any function.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    GRADIENT_FUNC_FIELD_NUMBER: builtins.int
    REGISTERED_OP_TYPE_FIELD_NUMBER: builtins.int
    gradient_func: builtins.str
    """The gradient function's name."""
    registered_op_type: builtins.str
    """The gradient function's registered op type."""
    def __init__(self, *, gradient_func: builtins.str | None = ..., registered_op_type: builtins.str | None = ...) -> None: ...
    def ClearField(
        self, field_name: typing.Literal["gradient_func", b"gradient_func", "registered_op_type", b"registered_op_type"]
    ) -> None: ...

global___RegisteredGradient = RegisteredGradient



from _typeshed import Unused
from collections.abc import Callable, Sequence
from io import TextIOWrapper
from typing import Final

HELP: Final[str]

def show_help(stream: TextIOWrapper, name: str, error: str | None = None) -> None: ...
def show_exception(stream: TextIOWrapper) -> None: ...
def run(argv: Sequence[str] = ..., _serve: Callable[..., Unused] = ...) -> None: ...



import datetime
from typing import ClassVar

from ..core import WesternCalendar

class Argentina(WesternCalendar):
    fat_tuesday_label: ClassVar[str]
    def get_general_guemes_day(self, year: int) -> tuple[datetime.date, str]: ...
    def get_general_martin_day(self, year: int) -> tuple[datetime.date, str]: ...
    def get_soberania_day(self, year: int) -> tuple[datetime.date, str]: ...
    def get_diversidad_day(self, year: int) -> tuple[datetime.date, str]: ...
    def get_malvinas_day(self, year: int) -> tuple[datetime.date, str]: ...



############################################################################
# Original work Copyright 2018 Palantir Technologies, Inc.                 #
# Original work licensed under the MIT License.                            #
# See ThirdPartyNotices.txt in the project root for license information.   #
# All modifications Copyright (c) Open Law Library. All rights reserved.   #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License")           #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#     http: // www.apache.org/licenses/LICENSE-2.0                         #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
import traceback
from typing import Set
from typing import Type
from lsprotocol.types import ResponseError


class JsonRpcException(Exception):
    """A class used as a base class for json rpc exceptions."""

    def __init__(self, message=None, code=None, data=None):
        message = message or getattr(self.__class__, "MESSAGE")
        super().__init__(message)
        self.message = message
        self.code = code or getattr(self.__class__, "CODE")
        self.data = data

    def __eq__(self, other):
        return (
            isinstance(other, self.__class__)
            and self.code == other.code
            and self.message == other.message
        )

    def __hash__(self):
        return hash((self.code, self.message))

    @staticmethod
    def from_error(error):
        for exc_class in _EXCEPTIONS:
            if exc_class.supports_code(error.code):
                return exc_class(
                    code=error.code, message=error.message, data=error.data
                )

        return JsonRpcException(code=error.code, message=error.message, data=error.data)

    @classmethod
    def supports_code(cls, code):
        # Defaults to UnknownErrorCode
        return getattr(cls, "CODE", -32001) == code

    def to_response_error(self) -> ResponseError:
        return ResponseError(code=self.code, message=self.message, data=self.data)


class JsonRpcInternalError(JsonRpcException):
    CODE = -32603
    MESSAGE = "Internal Error"

    @classmethod
    def of(cls, exc_info):
        exc_type, exc_value, exc_tb = exc_info
        return cls(
            message="".join(
                traceback.format_exception_only(exc_type, exc_value)
            ).strip(),
            data={"traceback": traceback.format_tb(exc_tb)},
        )


class JsonRpcInvalidParams(JsonRpcException):
    CODE = -32602
    MESSAGE = "Invalid Params"


class JsonRpcInvalidRequest(JsonRpcException):
    CODE = -32600
    MESSAGE = "Invalid Request"


class JsonRpcMethodNotFound(JsonRpcException):
    CODE = -32601
    MESSAGE = "Method Not Found"

    @classmethod
    def of(cls, method):
        return cls(message=cls.MESSAGE + ": " + method)


class JsonRpcParseError(JsonRpcException):
    CODE = -32700
    MESSAGE = "Parse Error"


class JsonRpcRequestCancelled(JsonRpcException):
    CODE = -32800
    MESSAGE = "Request Cancelled"


class JsonRpcContentModified(JsonRpcException):
    CODE = -32801
    MESSAGE = "Content Modified"


class JsonRpcServerNotInitialized(JsonRpcException):
    CODE = -32002
    MESSAGE = "ServerNotInitialized"


class JsonRpcUnknownErrorCode(JsonRpcException):
    CODE = -32001
    MESSAGE = "UnknownErrorCode"


class JsonRpcReservedErrorRangeStart(JsonRpcException):
    CODE = -32099
    MESSAGE = "jsonrpcReservedErrorRangeStart"


class JsonRpcReservedErrorRangeEnd(JsonRpcException):
    CODE = -32000
    MESSAGE = "jsonrpcReservedErrorRangeEnd"


class LspReservedErrorRangeStart(JsonRpcException):
    CODE = -32899
    MESSAGE = "lspReservedErrorRangeStart"


class LspReservedErrorRangeEnd(JsonRpcException):
    CODE = -32800
    MESSAGE = "lspReservedErrorRangeEnd"


class JsonRpcServerError(JsonRpcException):
    def __init__(self, message, code, data=None):
        if not _is_server_error_code(code):
            raise ValueError("Error code should be in range -32099 - -32000")
        super().__init__(message=message, code=code, data=data)

    @classmethod
    def supports_code(cls, code):
        return _is_server_error_code(code)


def _is_server_error_code(code):
    return -32099 <= code <= -32000


_EXCEPTIONS: Set[Type[JsonRpcException]] = {
    JsonRpcInternalError,
    JsonRpcInvalidParams,
    JsonRpcInvalidRequest,
    JsonRpcMethodNotFound,
    JsonRpcParseError,
    JsonRpcRequestCancelled,
    JsonRpcServerError,
}


class PyglsError(Exception):
    pass


class CommandAlreadyRegisteredError(PyglsError):
    def __init__(self, command_name):
        self.command_name = command_name

    def __repr__(self):
        return f'Command "{self.command_name}" is already registered.'


class FeatureAlreadyRegisteredError(PyglsError):
    def __init__(self, feature_name):
        self.feature_name = feature_name

    def __repr__(self):
        return f'Feature "{self.feature_name}" is already registered.'


class FeatureRequestError(PyglsError):
    pass


class FeatureNotificationError(PyglsError):
    pass


class MethodTypeNotRegisteredError(PyglsError):
    def __init__(self, name):
        self.name = name

    def __repr__(self):
        return f'"{self.name}" is not added to `pygls.lsp.LSP_METHODS_MAP`.'


class ThreadDecoratorError(PyglsError):
    pass


class ValidationError(PyglsError):
    def __init__(self, errors=None):
        self.errors = errors or []

    def __repr__(self):
        opt_errs = "\n-".join([e for e in self.errors])
        return f"Missing options: {opt_errs}"



from typing import Literal, overload

from pymeeus.Angle import Angle
from pymeeus.Epoch import Epoch

class JupiterMoons:
    @staticmethod
    def jupiter_system_angles(epoch: Epoch) -> tuple[float, float]: ...
    @staticmethod
    def rectangular_positions_jovian_equatorial(
        epoch: Epoch, tofk5: bool = True, solar: bool = False, do_correction: bool = True
    ) -> tuple[
        tuple[float, float, float], tuple[float, float, float], tuple[float, float, float], tuple[float, float, float]
    ]: ...
    @overload
    @staticmethod
    def apparent_rectangular_coordinates(
        epoch: Epoch,
        X: float,
        Y: float,
        Z: float,
        OMEGA: float,
        psi: float,
        i: float,
        lambda_0: float,
        beta_0: float,
        D: float = 0,
        isFictional: Literal[True] = ...,
    ) -> float: ...
    @overload
    @staticmethod
    def apparent_rectangular_coordinates(
        epoch: Epoch,
        X: float,
        Y: float,
        Z: float,
        OMEGA: float,
        psi: float,
        i: float,
        lambda_0: float,
        beta_0: float,
        D: float = 0,
        isFictional: Literal[False] | None = False,
    ) -> tuple[float, float, float]: ...
    @staticmethod
    def calculate_delta(
        epoch: Epoch,
    ) -> tuple[float, float, Angle, Angle, float] | tuple[float, float, Literal[0], Literal[0], Literal[0]]: ...
    @overload
    @staticmethod
    def correct_rectangular_positions(
        R: float, i_sat: int, DELTA: float, X_coordinate: list[float] | tuple[float, float, float]
    ) -> tuple[float, float, float]: ...
    @overload
    @staticmethod
    def correct_rectangular_positions(
        R: float, i_sat: int, DELTA: float, X_coordinate: float, Y_coordinate: float = 0, Z_coordinate: float = 0
    ) -> tuple[float, float, float]: ...
    @overload
    @staticmethod
    def check_phenomena(epoch: Epoch, check_all: Literal[True] = True, i_sat: int = 0) -> list[list[float]]: ...
    @overload
    @staticmethod
    def check_phenomena(epoch: Epoch, check_all: Literal[False] | None, i_sat: int = 0) -> tuple[float, float]: ...
    @staticmethod
    def is_phenomena(epoch: Epoch) -> list[list[bool]]: ...
    @staticmethod
    def check_coordinates(X: float, Y: float) -> float: ...
    @staticmethod
    def check_occultation(
        X: float = 0, Y: float = 0, Z: float = 0, epoch: Epoch | None = None, i_sat: int | None = None
    ) -> float: ...
    @staticmethod
    def check_eclipse(
        X_0: float = 0, Y_0: float = 0, Z_0: float = 0, epoch: Epoch | None = None, i_sat: int | None = None
    ) -> float: ...

def main() -> None: ...



from collections.abc import Iterator, Mapping as DictMixin
from typing import TypeVar

_T = TypeVar("_T")
_VT = TypeVar("_VT")

class LazyDict(DictMixin[str, _VT]):
    data: dict[str, _VT] | None
    def __getitem__(self, key: str) -> _VT: ...
    def __contains__(self, key: object) -> bool: ...
    def __iter__(self) -> Iterator[str]: ...
    def __len__(self) -> int: ...

class LazyList(list[_T]):
    # does not return `Self` type:
    def __new__(cls, fill_iter: _T | None = None) -> LazyList[_T]: ...

class LazySet(set[_T]):
    # does not return `Self` type:
    def __new__(cls, fill_iter: _T | None = None) -> LazySet[_T]: ...



from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin

class Audit(SystemBackendMixin):
    def list_enabled_audit_devices(self): ...
    def enable_audit_device(self, device_type, description=None, options=None, path=None, local=None): ...
    def disable_audit_device(self, path): ...
    def calculate_hash(self, path, input_to_hash): ...



import abc

from ...._types import Box
from ....main import ActiveWithNeighbors
from ...base import BaseImage

class QRModuleDrawer(abc.ABC, metaclass=abc.ABCMeta):
    needs_neighbors: bool = False
    img: BaseImage
    def initialize(self, img: BaseImage) -> None: ...
    @abc.abstractmethod
    def drawrect(self, box: Box, is_active: bool | ActiveWithNeighbors) -> None: ...



from _typeshed import Incomplete
from typing import Final

__version__: Final[str]

class PDFImage:
    image: Incomplete
    x: Incomplete
    y: Incomplete
    width: Incomplete
    height: Incomplete
    filename: Incomplete
    imageCaching: Incomplete
    colorSpace: str
    bitsPerComponent: int
    filters: Incomplete
    source: Incomplete
    def __init__(self, image, x, y, width=None, height=None, caching: int = 0) -> None: ...
    def jpg_imagedata(self): ...
    def cache_imagedata(self): ...
    def PIL_imagedata(self): ...
    def non_jpg_imagedata(self, image): ...
    imageData: Incomplete
    imgwidth: Incomplete
    imgheight: Incomplete
    def getImageData(self, preserveAspectRatio: bool = False) -> None: ...
    def drawInlineImage(
        self,
        canvas,
        preserveAspectRatio: bool = False,
        anchor: str = "sw",
        anchorAtXY: bool = False,
        showBoundary: bool = False,
        extraReturn=None,
    ): ...
    def format(self, document): ...



from typing import ClassVar, Final

from docutils import nodes
from docutils.utils import Reporter
from docutils.writers import latex2e

__docformat__: Final = "reStructuredText"

class Writer(latex2e.Writer):
    default_template: ClassVar[str]
    default_preamble: ClassVar[str]
    config_section: ClassVar[str]
    config_section_dependencies: ClassVar[tuple[str, ...]]
    translator_class: type[XeLaTeXTranslator]

class Babel(latex2e.Babel):
    language_code: str
    reporter: Reporter
    language: str
    warn_msg: str  # type: ignore[misc]
    quote_index: int
    quotes: tuple[str, ...]
    literal_double_quote: str
    key: str
    def __init__(self, language_code: str, reporter: Reporter) -> None: ...

class XeLaTeXTranslator(latex2e.LaTeXTranslator):
    is_xetex: bool  # type: ignore[misc]
    def __init__(self, document: nodes.document) -> None: ...
    def to_latex_length(self, length_str: str, node: nodes.Node | None = None) -> str: ...



from _typeshed import Incomplete
from collections.abc import Generator
from typing import ClassVar, Final, TextIO, TypeVar

_T = TypeVar("_T")

__docformat__: Final = "reStructuredText"
__version__: Final[str]

class Trace:
    debugmode: ClassVar[bool]
    quietmode: ClassVar[bool]
    showlinesmode: ClassVar[bool]
    prefix: ClassVar[str | None]
    @classmethod
    def debug(cls, message: str) -> None: ...
    @classmethod
    def message(cls, message: str) -> None: ...
    @classmethod
    def error(cls, message: str) -> None: ...
    @classmethod
    def show(cls, message: str, channel: TextIO) -> None: ...

class ContainerConfig:
    extracttext: ClassVar[dict[str, list[str]]]

class EscapeConfig:
    chars: ClassVar[dict[str, str]]
    entities: ClassVar[dict[str, str]]

class FormulaConfig:
    alphacommands: ClassVar[dict[str, str]]
    array: ClassVar[dict[str, str]]
    bigbrackets: ClassVar[dict[str, list[str]]]
    bracketcommands: ClassVar[dict[str, str]]
    combiningfunctions: ClassVar[dict[str, str]]
    commands: ClassVar[dict[str, str]]
    cmddict: ClassVar[dict[str, str]]
    oversetfunctions: ClassVar[dict[str, str]]
    undersetfunctions: ClassVar[dict[str, str]]
    endings: ClassVar[dict[str, str]]
    environments: ClassVar[dict[str, list[str]]]
    fontfunctions: ClassVar[dict[str, str]]
    hybridfunctions: ClassVar[dict[str, list[str]]]
    hybridsizes: ClassVar[dict[str, str]]
    labelfunctions: ClassVar[dict[str, str]]
    limitcommands: ClassVar[dict[str, str]]
    modified: ClassVar[dict[str, str]]
    onefunctions: ClassVar[dict[str, str]]
    spacedcommands: ClassVar[dict[str, str]]
    starts: ClassVar[dict[str, str]]
    symbolfunctions: ClassVar[dict[str, str]]
    textfunctions: ClassVar[dict[str, str]]
    unmodified: ClassVar[dict[str, list[str]]]
    key: str
    value: str

class CommandLineParser:
    options: Incomplete
    def __init__(self, options) -> None: ...
    def parseoptions(self, args): ...
    def readoption(self, args): ...
    def readquoted(self, args, initial): ...
    def readequalskey(self, arg, args): ...

class Options:
    location: Incomplete
    debug: bool
    quiet: bool
    version: bool
    help: bool
    simplemath: bool
    showlines: bool
    branches: Incomplete
    def parseoptions(self, args) -> None: ...
    def processoptions(self) -> None: ...
    def usage(self) -> None: ...
    def showoptions(self) -> None: ...
    def showversion(self) -> None: ...

class Cloner:
    @classmethod
    def clone(cls, original: _T) -> _T: ...
    @classmethod
    def create(cls, type: type[_T]) -> _T: ...

class ContainerExtractor:
    allowed: Incomplete
    extracted: Incomplete
    def __init__(self, config) -> None: ...
    def extract(self, container): ...
    def process(self, container, lst) -> None: ...
    def safeclone(self, container): ...

class Parser:
    begin: int
    parameters: Incomplete
    def __init__(self) -> None: ...
    def parseheader(self, reader): ...
    def parseparameter(self, reader) -> None: ...
    def parseending(self, reader, process) -> None: ...
    def parsecontainer(self, reader, contents) -> None: ...

class LoneCommand(Parser):
    def parse(self, reader): ...

class TextParser(Parser):
    stack: Incomplete
    ending: Incomplete
    endings: Incomplete
    def __init__(self, container) -> None: ...
    def parse(self, reader): ...
    def isending(self, reader): ...

class ExcludingParser(Parser):
    def parse(self, reader): ...

class BoundedParser(ExcludingParser):
    def parse(self, reader): ...

class BoundedDummy(Parser):
    def parse(self, reader): ...

class StringParser(Parser):
    begin: Incomplete
    def parseheader(self, reader): ...
    def parse(self, reader): ...

class ContainerOutput:
    def gethtml(self, container) -> None: ...
    def isempty(self): ...

class EmptyOutput(ContainerOutput):
    def gethtml(self, container): ...
    def isempty(self): ...

class FixedOutput(ContainerOutput):
    def gethtml(self, container): ...

class ContentsOutput(ContainerOutput):
    def gethtml(self, container): ...

class TaggedOutput(ContentsOutput):
    tag: Incomplete
    breaklines: bool
    empty: bool
    def settag(self, tag, breaklines: bool = False, empty: bool = False): ...
    def setbreaklines(self, breaklines): ...
    def gethtml(self, container): ...
    def open(self, container): ...
    def close(self, container): ...
    def selfclosing(self, container): ...
    def checktag(self, container): ...

class FilteredOutput(ContentsOutput):
    filters: Incomplete
    def __init__(self) -> None: ...
    def addfilter(self, original, replacement) -> None: ...
    def gethtml(self, container): ...
    def filter(self, line): ...

class StringOutput(ContainerOutput):
    def gethtml(self, container): ...

class Globable:
    leavepending: bool
    endinglist: Incomplete
    def __init__(self) -> None: ...
    def checkbytemark(self) -> None: ...
    def isout(self): ...
    def current(self): ...
    def checkfor(self, string): ...
    def finished(self): ...
    def skipcurrent(self): ...
    def glob(self, currentcheck): ...
    def globalpha(self): ...
    def globnumber(self): ...
    def isidentifier(self): ...
    def globidentifier(self): ...
    def isvalue(self): ...
    def globvalue(self): ...
    def skipspace(self): ...
    def globincluding(self, magicchar): ...
    def globexcluding(self, excluded): ...
    def pushending(self, ending, optional: bool = False) -> None: ...
    def popending(self, expected=None): ...
    def nextending(self): ...

class EndingList:
    endings: Incomplete
    def __init__(self) -> None: ...
    def add(self, ending, optional: bool = False) -> None: ...
    def pickpending(self, pos) -> None: ...
    def checkin(self, pos): ...
    def pop(self, pos): ...
    def findending(self, pos): ...
    def checkpending(self) -> None: ...

class PositionEnding:
    ending: Incomplete
    optional: Incomplete
    def __init__(self, ending, optional) -> None: ...
    def checkin(self, pos): ...

class Position(Globable):
    def __init__(self) -> None: ...
    def skip(self, string) -> None: ...
    def identifier(self): ...
    def extract(self, length) -> None: ...
    def checkfor(self, string): ...
    def checkforlower(self, string): ...
    def skipcurrent(self): ...
    def __next__(self): ...
    def checkskip(self, string): ...
    def error(self, message) -> None: ...

class TextPosition(Position):
    pos: int
    text: Incomplete
    def __init__(self, text) -> None: ...
    def skip(self, string) -> None: ...
    def identifier(self): ...
    def isout(self): ...
    def current(self): ...
    def extract(self, length): ...

class Container:
    partkey: Incomplete
    parent: Incomplete
    begin: Incomplete
    contents: Incomplete
    def __init__(self) -> None: ...
    def process(self) -> None: ...
    def gethtml(self): ...
    def escape(self, line, replacements={"&": "&amp;", "<": "&lt;", ">": "&gt;"}): ...
    def escapeentities(self, line): ...
    def searchall(self, type): ...
    def searchremove(self, type): ...
    def searchprocess(self, type, process): ...
    def locateprocess(self, locate, process) -> None: ...
    def recursivesearch(self, locate, recursive, process) -> None: ...
    def extracttext(self): ...
    def group(self, index, group, isingroup) -> None: ...
    def remove(self, index) -> None: ...
    def tree(self, level: int = 0) -> None: ...
    def getparameter(self, name): ...
    def getparameterlist(self, name): ...
    def hasemptyoutput(self): ...

class BlackBox(Container):
    parser: Incomplete
    output: Incomplete
    contents: Incomplete
    def __init__(self) -> None: ...

class StringContainer(Container):
    parsed: Incomplete
    parser: Incomplete
    output: Incomplete
    string: str
    def __init__(self) -> None: ...
    def process(self) -> None: ...
    def replacespecial(self, line): ...
    def changeline(self, line): ...
    def extracttext(self): ...

class Constant(StringContainer):
    contents: Incomplete
    string: Incomplete
    output: Incomplete
    def __init__(self, text) -> None: ...

class DocumentParameters:
    displaymode: bool

class FormulaParser(Parser):
    begin: Incomplete
    def parseheader(self, reader): ...
    def parsetype(self, reader): ...
    def parse(self, reader): ...
    def parseformula(self, reader): ...
    def parsesingleliner(self, reader, start, ending): ...
    def parsemultiliner(self, reader, start, ending): ...

class FormulaBit(Container):
    type: str | None
    size: int
    original: str
    contents: Incomplete
    output: Incomplete
    def __init__(self) -> None: ...
    factory: Incomplete
    def setfactory(self, factory): ...
    def add(self, bit) -> None: ...
    def skiporiginal(self, string, pos) -> None: ...
    def computesize(self): ...
    def clone(self): ...

class TaggedBit(FormulaBit):
    output: Incomplete
    def constant(self, constant, tag): ...
    contents: Incomplete
    def complete(self, contents, tag, breaklines: bool = False): ...
    def selfcomplete(self, tag): ...

class FormulaConstant(Constant):
    original: Incomplete
    size: int
    type: str | None
    def __init__(self, string) -> None: ...
    def computesize(self): ...
    def clone(self): ...

class RawText(FormulaBit):
    def detect(self, pos): ...
    def parsebit(self, pos) -> None: ...

class FormulaSymbol(FormulaBit):
    modified: Incomplete
    unmodified: Incomplete
    def detect(self, pos): ...
    def parsebit(self, pos) -> None: ...
    def addsymbol(self, symbol, pos) -> None: ...

class FormulaNumber(FormulaBit):
    def detect(self, pos): ...
    def parsebit(self, pos): ...

class Comment(FormulaBit):
    start: Incomplete
    def detect(self, pos): ...
    def parsebit(self, pos) -> None: ...

class WhiteSpace(FormulaBit):
    def detect(self, pos): ...
    def parsebit(self, pos) -> None: ...

class Bracket(FormulaBit):
    start: Incomplete
    ending: Incomplete
    inner: Incomplete
    def __init__(self) -> None: ...
    def detect(self, pos): ...
    def parsebit(self, pos): ...
    def parsetext(self, pos): ...
    def parseliteral(self, pos): ...
    def parsecomplete(self, pos, innerparser) -> None: ...
    def innerformula(self, pos) -> None: ...
    def innertext(self, pos) -> None: ...
    literal: str
    def innerliteral(self, pos) -> None: ...

class SquareBracket(Bracket):
    start: Incomplete
    ending: Incomplete
    def clone(self): ...

class MathsProcessor:
    def process(self, contents, index) -> None: ...

class FormulaProcessor:
    processors: Incomplete
    def process(self, bit) -> None: ...
    def processcontents(self, bit) -> None: ...
    def processinsides(self, bit) -> None: ...
    def traversewhole(self, formula) -> None: ...
    def traverse(self, bit) -> Generator[Incomplete, Incomplete]: ...
    def italicize(self, bit, contents) -> None: ...

class Formula(Container):
    parser: Incomplete
    output: Incomplete
    def __init__(self) -> None: ...
    def process(self) -> None: ...
    contents: Incomplete
    def classic(self) -> None: ...
    def parse(self, pos): ...
    header: Incomplete
    def parsedollarinline(self, pos) -> None: ...
    def parsedollarblock(self, pos) -> None: ...
    parsed: Incomplete
    def parsedollar(self, pos) -> None: ...
    def parseinlineto(self, pos, limit) -> None: ...
    def parseblockto(self, pos, limit) -> None: ...
    def parseupto(self, pos, limit): ...

class WholeFormula(FormulaBit):
    def detect(self, pos): ...
    def parsebit(self, pos) -> None: ...

class FormulaFactory:
    types: Incomplete
    skippedtypes: Incomplete
    defining: bool
    instances: Incomplete
    def __init__(self) -> None: ...
    def detecttype(self, type, pos): ...
    def instance(self, type): ...
    def create(self, type): ...
    def clearskipped(self, pos) -> None: ...
    def skipany(self, pos): ...
    def parseany(self, pos): ...
    def parsetype(self, type, pos): ...
    def parseformula(self, formula): ...

class FormulaCommand(FormulaBit):
    types: Incomplete
    start: Incomplete
    commandmap: ClassVar[dict[str, str] | dict[str, list[str]] | None]
    def detect(self, pos): ...
    output: Incomplete
    def parsebit(self, pos): ...
    def parsewithcommand(self, command, pos): ...
    def parsecommandtype(self, command, type, pos): ...
    def extractcommand(self, pos): ...
    def emptycommand(self, pos): ...
    def parseupgreek(self, command, pos): ...

class CommandBit(FormulaCommand):
    command: Incomplete
    translated: Incomplete
    def setcommand(self, command) -> None: ...
    def parseparameter(self, pos): ...
    def parsesquare(self, pos): ...
    def parseliteral(self, pos): ...
    def parsesquareliteral(self, pos): ...
    def parsetext(self, pos): ...

class EmptyCommand(CommandBit):
    commandmap: ClassVar[dict[str, str]]
    contents: Incomplete
    def parsebit(self, pos) -> None: ...

class SpacedCommand(CommandBit):
    commandmap: ClassVar[dict[str, str]]
    contents: Incomplete
    def parsebit(self, pos) -> None: ...

class AlphaCommand(EmptyCommand):
    commandmap: ClassVar[dict[str, str]]
    greek_capitals: Incomplete
    def parsebit(self, pos) -> None: ...

class OneParamFunction(CommandBit):
    commandmap: ClassVar[dict[str, str]]
    simplified: bool
    output: Incomplete
    def parsebit(self, pos) -> None: ...
    html: Incomplete
    def simplifyifpossible(self) -> None: ...

class SymbolFunction(CommandBit):
    commandmap: ClassVar[dict[str, str]]
    def detect(self, pos): ...
    output: Incomplete
    def parsebit(self, pos) -> None: ...

class TextFunction(CommandBit):
    commandmap: ClassVar[dict[str, str]]
    output: Incomplete
    def parsebit(self, pos) -> None: ...
    def process(self) -> None: ...

class FontFunction(OneParamFunction):
    commandmap: ClassVar[dict[str, str]]
    def process(self) -> None: ...

class BigBracket:
    size: Incomplete
    original: Incomplete
    alignment: Incomplete
    pieces: Incomplete
    def __init__(self, size, bracket, alignment: str = "l") -> None: ...
    def getpiece(self, index): ...
    def getpiece1(self, index): ...
    def getpiece3(self, index): ...
    def getpiece4(self, index): ...
    def getcell(self, index): ...
    def getcontents(self): ...
    def getsinglebracket(self): ...

class FormulaEquation(CommandBit):
    piece: str
    output: Incomplete
    def parsebit(self, pos) -> None: ...

class FormulaCell(FormulaCommand):
    alignment: Incomplete
    output: Incomplete
    def setalignment(self, alignment): ...
    def parsebit(self, pos) -> None: ...

class FormulaRow(FormulaCommand):
    cellseparator: Incomplete
    alignments: Incomplete
    output: Incomplete
    def setalignments(self, alignments): ...
    def parsebit(self, pos) -> None: ...
    def createcell(self, index): ...

class MultiRowFormula(CommandBit):
    rows: Incomplete
    size: Incomplete
    def parserows(self, pos) -> None: ...
    def iteraterows(self, pos) -> Generator[Incomplete]: ...
    def addempty(self) -> None: ...
    def addrow(self, row) -> None: ...

class FormulaArray(MultiRowFormula):
    piece: str
    output: Incomplete
    def parsebit(self, pos) -> None: ...
    valign: str
    alignments: Incomplete
    def parsealignments(self, pos) -> None: ...

class FormulaMatrix(MultiRowFormula):
    piece: str
    output: Incomplete
    valign: str
    alignments: Incomplete
    def parsebit(self, pos) -> None: ...

class FormulaCases(MultiRowFormula):
    piece: str
    output: Incomplete
    alignments: Incomplete
    contents: Incomplete
    def parsebit(self, pos) -> None: ...

class EquationEnvironment(MultiRowFormula):
    output: Incomplete
    alignments: Incomplete
    def parsebit(self, pos) -> None: ...

class BeginCommand(CommandBit):
    commandmap: ClassVar[dict[str, str]]
    types: Incomplete
    size: Incomplete
    def parsebit(self, pos) -> None: ...
    def findbit(self, piece): ...

class CombiningFunction(OneParamFunction):
    commandmap: ClassVar[dict[str, str]]
    def parsebit(self, pos) -> None: ...
    def parsesingleparameter(self, pos): ...

class OversetFunction(OneParamFunction):
    commandmap: ClassVar[dict[str, str]]
    symbol: Incomplete
    parameter: Incomplete
    output: Incomplete
    def parsebit(self, pos) -> None: ...

class UndersetFunction(OneParamFunction):
    commandmap: ClassVar[dict[str, str]]
    symbol: Incomplete
    parameter: Incomplete
    output: Incomplete
    def parsebit(self, pos) -> None: ...

class LimitCommand(EmptyCommand):
    commandmap: ClassVar[dict[str, str]]
    output: Incomplete
    def parsebit(self, pos) -> None: ...

class LimitPreviousCommand(LimitCommand):
    commandmap: ClassVar[None]  # type: ignore[assignment]
    output: Incomplete
    def parsebit(self, pos) -> None: ...

class LimitsProcessor(MathsProcessor):
    def process(self, contents, index) -> None: ...
    def checklimits(self, contents, index): ...
    def limitsahead(self, contents, index) -> None: ...
    def modifylimits(self, contents, index) -> None: ...
    def getlimit(self, contents, index): ...
    def modifyscripts(self, contents, index) -> None: ...
    def checkscript(self, contents, index): ...
    def checkcommand(self, contents, index, type): ...
    def getscript(self, contents, index): ...

class BracketCommand(OneParamFunction):
    commandmap: ClassVar[dict[str, str]]
    def parsebit(self, pos) -> None: ...
    original: Incomplete
    command: Incomplete
    contents: Incomplete
    def create(self, direction, character): ...

class BracketProcessor(MathsProcessor):
    def process(self, contents, index): ...
    def processleft(self, contents, index) -> None: ...
    def checkleft(self, contents, index): ...
    def checkright(self, contents, index): ...
    def checkdirection(self, bit, command): ...
    def findright(self, contents, index): ...
    def findmax(self, contents, leftindex, rightindex): ...
    def resize(self, command, size) -> None: ...

class ParameterDefinition:
    parambrackets: ClassVar[list[tuple[str, str]]]
    name: str
    literal: bool
    optional: bool
    value: Incomplete
    literalvalue: Incomplete
    def __init__(self) -> None: ...
    def parse(self, pos): ...
    def read(self, pos, function) -> None: ...

class ParameterFunction(CommandBit):
    params: Incomplete
    def readparams(self, readtemplate, pos) -> None: ...
    def paramdefs(self, readtemplate) -> Generator[Incomplete]: ...
    def getparam(self, name): ...
    def getvalue(self, name): ...
    def getliteralvalue(self, name): ...

class HybridFunction(ParameterFunction):
    commandmap: ClassVar[dict[str, list[str]]]
    contents: Incomplete
    def parsebit(self, pos) -> None: ...
    def writeparams(self, writetemplate): ...
    def writepos(self, pos): ...
    def writeparam(self, pos): ...
    def writefunction(self, pos): ...
    def readtag(self, pos): ...
    def writebracket(self, direction, character): ...
    size: Incomplete
    def computehybridsize(self) -> None: ...

class HybridSize:
    configsizes: ClassVar[dict[str, str]]
    def getsize(self, function): ...

def math2html(formula): ...
def main() -> None: ...



from _typeshed import Incomplete

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["is_at_free", "find_asteroidal_triple"]

@_dispatchable
def find_asteroidal_triple(G: Graph[_Node]) -> list[Incomplete] | None: ...
@_dispatchable
def is_at_free(G: Graph[_Node]) -> bool: ...
@_dispatchable
def create_component_structure(G: Graph[_Node]) -> dict[Incomplete, Incomplete]: ...



from _typeshed import Incomplete
from typing import NamedTuple

class DeleteNode(NamedTuple):
    node: str

class InsertNode(NamedTuple):
    target: Incomplete
    tag: str
    position: int

class RenameNode(NamedTuple):
    node: str
    tag: Incomplete

class MoveNode(NamedTuple):
    node: str
    target: Incomplete
    position: int

class UpdateTextIn(NamedTuple):
    node: str
    text: Incomplete

class UpdateTextAfter(NamedTuple):
    node: str
    text: Incomplete

class UpdateAttrib(NamedTuple):
    node: str
    name: str
    value: Incomplete

class DeleteAttrib(NamedTuple):
    node: str
    name: str

class InsertAttrib(NamedTuple):
    node: str
    name: str
    value: Incomplete

class RenameAttrib(NamedTuple):
    node: str
    oldname: str
    newname: str

class InsertComment(NamedTuple):
    target: Incomplete
    position: Incomplete
    text: Incomplete

class InsertNamespace(NamedTuple):
    prefix: str
    uri: str

class DeleteNamespace(NamedTuple):
    prefix: str



from _typeshed import Incomplete
from typing import Final

from reportlab.graphics.widgetbase import PropHolder, Widget
from reportlab.lib.attrmap import *

__version__: Final[str]

class SubColProperty(PropHolder):
    dividerLines: int

class LegendCallout:
    def __call__(self, legend, g, thisx, y, colName) -> None: ...

class LegendSwatchCallout(LegendCallout):
    def __call__(self, legend, g, thisx, y, i, colName, swatch) -> None: ...  # type: ignore[override]

class LegendColEndCallout(LegendCallout):
    def __call__(self, legend, g, x, xt, y, width, lWidth) -> None: ...  # type: ignore[override]

class Legend(Widget):
    x: int
    y: int
    alignment: str
    deltax: int
    deltay: int
    autoXPadding: int
    autoYPadding: int
    dx: int
    dy: int
    swdx: int
    swdy: int
    dxTextSpace: int
    columnMaximum: int
    colorNamePairs: Incomplete
    fontName: Incomplete
    fontSize: Incomplete
    leading: Incomplete
    fillColor: Incomplete
    strokeColor: Incomplete
    strokeWidth: Incomplete
    swatchMarker: Incomplete
    boxAnchor: str
    yGap: int
    variColumn: int
    dividerLines: int
    dividerWidth: float
    dividerDashArray: Incomplete
    dividerColor: Incomplete
    dividerOffsX: Incomplete
    dividerOffsY: int
    colEndCallout: Incomplete
    def __init__(self) -> None: ...
    def draw(self): ...
    def demo(self): ...

class TotalAnnotator(LegendColEndCallout):
    lText: Incomplete
    rText: Incomplete
    fontName: Incomplete
    fontSize: Incomplete
    fillColor: Incomplete
    dy: Incomplete
    dx: Incomplete
    dly: Incomplete
    dlx: Incomplete
    strokeWidth: Incomplete
    strokeColor: Incomplete
    strokeDashArray: Incomplete
    def __init__(
        self,
        lText: str = "Total",
        rText: str = "0.0",
        fontName="Times-Roman",
        fontSize: int = 10,
        fillColor=...,
        strokeWidth: float = 0.5,
        strokeColor=...,
        strokeDashArray=None,
        dx: int = 0,
        dy: int = 0,
        dly: int = 0,
        dlx=(0, 0),
    ) -> None: ...
    def __call__(self, legend, g, x, xt, y, width, lWidth) -> None: ...  # type: ignore[override]

class LineSwatch(Widget):
    x: int
    y: int
    width: int
    height: int
    strokeColor: Incomplete
    strokeDashArray: Incomplete
    strokeWidth: int
    def __init__(self) -> None: ...
    def draw(self): ...

class LineLegend(Legend):
    dx: int
    dy: int
    def __init__(self) -> None: ...



from _typeshed import Incomplete
from dataclasses import dataclass
from typing import Any, TypeVar
from typing_extensions import TypeAlias

from numpy.typing import NDArray
from pandas import DataFrame
from seaborn._core.properties import DashPattern, DashPatternWithOffset, RGBATuple
from seaborn._core.scales import Scale

_MarkT = TypeVar("_MarkT", bound=type[Mark])

class Mappable:
    def __init__(
        self, val: Any = None, depend: str | None = None, rc: str | None = None, auto: bool = False, grouping: bool = True
    ) -> None: ...
    @property
    def depend(self) -> Any: ...  # -> str | None
    @property
    def grouping(self) -> bool: ...
    @property
    def default(self) -> Any: ...

MappableBool: TypeAlias = bool | Mappable
MappableString: TypeAlias = str | Mappable
MappableFloat: TypeAlias = float | Mappable
MappableColor: TypeAlias = str | tuple[Incomplete, ...] | Mappable
MappableStyle: TypeAlias = str | DashPattern | DashPatternWithOffset | Mappable

@dataclass
class Mark:
    artist_kws: dict[str, Any] = ...

def resolve_properties(mark: Mark, data: DataFrame, scales: dict[str, Scale]) -> dict[str, Any]: ...
def resolve_color(
    mark: Mark, data: DataFrame | dict[str, Any], prefix: str = "", scales: dict[str, Scale] | None = None
) -> RGBATuple | NDArray[Incomplete]: ...
def document_properties(mark: _MarkT) -> _MarkT: ...
